@@ -59,15 +59,18 @@ public class ConnectionFactory {
   }
 
   /**
-   * Creates a connection to Pinot cluster, given its Controller URL
+   * Creates a connection to a Pinot cluster, given its Zookeeper URL and properties.
    *
-   * @param scheme controller URL scheme
-   * @param controllerHost controller host
-   * @param controllerPort controller port
-   * @return A connection that connects to brokers as per the given controller
+   * @param properties connection properties
+   * @param zkUrl zookeeper URL.
+   * @return A connection that connects to the brokers in the given Helix cluster
    */
-  public static Connection fromController(String scheme, String controllerHost, int controllerPort) {
-    return fromController(scheme, controllerHost, controllerPort, 1000);
+  public static Connection fromZookeeper(Properties properties, String zkUrl) {
+    try {
+      return fromZookeeper(properties, new DynamicBrokerSelector(zkUrl), getDefault(properties));
+    } catch (Exception e) {
+      throw new PinotClientException(e);
+    }
   }
 
   /**
@@ -75,20 +78,54 @@ public class ConnectionFactory {
    * @param scheme controller URL scheme
    * @param controllerHost controller host
    * @param controllerPort controller port
-   * @param brokerUpdateFreqInMillis frequency of broker data refresh using controller APIs
    * @return A connection that connects to brokers as per the given controller
    */
-  public static Connection fromController(String scheme, String controllerHost, int controllerPort,
-      long brokerUpdateFreqInMillis) {
+  @Deprecated
+  public static Connection fromController(String scheme, String controllerHost, int controllerPort) {
+    return fromController(new Properties(), scheme, controllerHost, controllerPort);
+  }
+
+  /**
+   *
+   * @param scheme controller URL scheme
+   * @param controllerHost controller host
+   * @param controllerPort controller port
+   * @return A connection that connects to brokers as per the given controller
+   */
+  @Deprecated
+  public static Connection fromController(Properties properties, String scheme,
+      String controllerHost, int controllerPort) {
     try {
-      return new Connection(new Properties(),
-          new ControllerBasedBrokerSelector(scheme, controllerHost, controllerPort, brokerUpdateFreqInMillis),
+      return new Connection(properties,
+          new ControllerBasedBrokerSelector(scheme, controllerHost, controllerPort, properties),
           getDefault());
     } catch (Exception e) {
       throw new PinotClientException(e);
     }
   }
 
+  /**
+   * @param controllerUrl url host:port of the controller
+   * @return A connection that connects to brokers as per the given controller
+   */
+  public static Connection fromController(String controllerUrl) {
+    return fromController(new Properties(), controllerUrl);
+  }
+
+  /**
+   * @param properties
+   * @param controllerUrl url host:port of the controller
+   * @return A connection that connects to brokers as per the given controller
+   */
+  public static Connection fromController(Properties properties, String controllerUrl) {
+    try {
+      return new Connection(properties,
+          new ControllerBasedBrokerSelector(properties, controllerUrl),
+          getDefault(properties));
+    } catch (Exception e) {
+      throw new PinotClientException(e);
+    }
+  }
   /**
    * Creates a connection to a Pinot cluster, given its Zookeeper URL
    *
