@@ -22,6 +22,7 @@ import java.util.HashMap;
 import java.util.Map;
 import java.util.Properties;
 import javax.net.ssl.SSLContext;
+import org.apache.pinot.client.utils.ConnectionUtils;
 import org.apache.pinot.spi.utils.CommonConstants;
 
 
@@ -42,14 +43,18 @@ public class JsonAsyncHttpPinotClientTransportFactory implements PinotClientTran
   private int _readTimeoutMs = Integer.parseInt(DEFAULT_BROKER_READ_TIMEOUT_MS);
   private int _connectTimeoutMs = Integer.parseInt(DEFAULT_BROKER_READ_TIMEOUT_MS);
   private int _handshakeTimeoutMs = Integer.parseInt(DEFAULT_BROKER_HANDSHAKE_TIMEOUT_MS);
+  private String _appId = null;
+  private String _extraOptionString;
 
   @Override
   public PinotClientTransport buildTransport() {
-    ConnectionTimeouts connectionTimeouts = ConnectionTimeouts.create(_readTimeoutMs, _connectTimeoutMs,
-            _handshakeTimeoutMs);
+    ConnectionTimeouts connectionTimeouts =
+        ConnectionTimeouts.create(_readTimeoutMs, _connectTimeoutMs, _handshakeTimeoutMs);
     TlsProtocols tlsProtocols = TlsProtocols.defaultProtocols(_tlsV10Enabled);
-    return new JsonAsyncHttpPinotClientTransport(_headers, _scheme, _sslContext, connectionTimeouts, tlsProtocols);
+    return new JsonAsyncHttpPinotClientTransport(_headers, _scheme, _extraOptionString, _sslContext, connectionTimeouts,
+        tlsProtocols, _appId);
   }
+
   public Map<String, String> getHeaders() {
     return _headers;
   }
@@ -75,16 +80,29 @@ public class JsonAsyncHttpPinotClientTransportFactory implements PinotClientTran
   }
 
   public JsonAsyncHttpPinotClientTransportFactory withConnectionProperties(Properties properties) {
-    _readTimeoutMs = Integer.parseInt(properties.getProperty("brokerReadTimeoutMs",
-            DEFAULT_BROKER_READ_TIMEOUT_MS));
-    _connectTimeoutMs = Integer.parseInt(properties.getProperty("brokerConnectTimeoutMs",
-            DEFAULT_BROKER_CONNECT_TIMEOUT_MS));
-    _handshakeTimeoutMs = Integer.parseInt(properties.getProperty("brokerHandshakeTimeoutMs",
-            DEFAULT_BROKER_HANDSHAKE_TIMEOUT_MS));
-    _tlsV10Enabled = Boolean.parseBoolean(properties.getProperty("brokerTlsV10Enabled",
-            DEFAULT_BROKER_TLS_V10_ENABLED))
-            || Boolean.parseBoolean(System.getProperties().getProperty("broker.tlsV10Enabled",
-            DEFAULT_BROKER_TLS_V10_ENABLED));
+    if (_headers == null || _headers.isEmpty()) {
+      _headers = ConnectionUtils.getHeadersFromProperties(properties);
+    }
+
+    if (_scheme == null) {
+      _scheme = properties.getProperty("scheme", CommonConstants.HTTP_PROTOCOL);
+    }
+
+    if (_sslContext == null && _scheme.contentEquals(CommonConstants.HTTPS_PROTOCOL)) {
+      _sslContext = ConnectionUtils.getSSLContextFromProperties(properties);
+    }
+
+    _readTimeoutMs = Integer.parseInt(properties.getProperty("brokerReadTimeoutMs", DEFAULT_BROKER_READ_TIMEOUT_MS));
+    _connectTimeoutMs =
+        Integer.parseInt(properties.getProperty("brokerConnectTimeoutMs", DEFAULT_BROKER_CONNECT_TIMEOUT_MS));
+    _handshakeTimeoutMs =
+        Integer.parseInt(properties.getProperty("brokerHandshakeTimeoutMs", DEFAULT_BROKER_HANDSHAKE_TIMEOUT_MS));
+    _appId = properties.getProperty("appId");
+    _tlsV10Enabled = Boolean.parseBoolean(properties.getProperty("brokerTlsV10Enabled", DEFAULT_BROKER_TLS_V10_ENABLED))
+        || Boolean.parseBoolean(
+        System.getProperties().getProperty("broker.tlsV10Enabled", DEFAULT_BROKER_TLS_V10_ENABLED));
+
+    _extraOptionString = properties.getProperty("queryOptions", "");
     return this;
   }
 }
