@@ -22,6 +22,7 @@ import com.fasterxml.jackson.core.JsonParseException;
 import java.math.BigDecimal;
 import java.sql.Timestamp;
 import java.util.Base64;
+import java.util.Collection;
 import org.apache.commons.lang3.ArrayUtils;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
 import org.apache.pinot.spi.data.FieldSpec;
@@ -923,6 +924,8 @@ public enum PinotDataType {
     }
   },
 
+  COLLECTION,
+
   OBJECT_ARRAY;
 
   /**
@@ -966,7 +969,7 @@ public enum PinotDataType {
     if (value instanceof String) {
       try {
         // Try to parse the string as JSON first
-        return JsonUtils.stringToJsonNode((String) value).toString();
+        return JsonUtils.stringToJsonNodeWithBigDecimal((String) value).toString();
       } catch (JsonParseException jpe) {
         // String does not represent a well-formed JSON. Ignore this exception because we are going to try to convert
         // Java String object to JSON string.
@@ -1201,6 +1204,9 @@ public enum PinotDataType {
   }
 
   private static Object[] toObjectArray(Object array) {
+    if (array instanceof Collection) {
+      return ((Collection<?>) array).toArray();
+    }
     Class<?> componentType = array.getClass().getComponentType();
     if (componentType.isPrimitive()) {
       if (componentType == int.class) {
@@ -1301,6 +1307,7 @@ public enum PinotDataType {
       case BYTES_ARRAY:
         return BYTES;
       case OBJECT_ARRAY:
+      case COLLECTION:
         return OBJECT;
       case BOOLEAN_ARRAY:
         return BOOLEAN;
