@@ -42,6 +42,7 @@ object ExampleSparkPinotConnectorTest extends Logging {
     readHybridWithFilters()
     readHybridViaGrpc()
     readRealtimeViaGrpc()
+    readRealtimeWithFilterViaGrpc()
     readHybridWithFiltersViaGrpc()
     readRealtimeWithSelectionColumns()
     applyJustSomeFilters()
@@ -163,6 +164,21 @@ object ExampleSparkPinotConnectorTest extends Logging {
     data.show()
   }
 
+  def readRealtimeWithFilterViaGrpc()(implicit spark: SparkSession): Unit = {
+    import spark.implicits._
+    log.info("## Reading `airlineStats_REALTIME` table... ##")
+    val data = spark.read
+      .format("pinot")
+      .option("table", "airlineStats")
+      .option("tableType", "realtime")
+      .option("useGrpcServer", "true")
+      .load()
+      .filter($"DestWac" === 5)
+      .select($"FlightNum", $"Origin", $"DestStateName")
+
+    data.show()
+  }
+
   def readHybridWithFiltersViaGrpc()(implicit spark: SparkSession): Unit = {
     import spark.implicits._
     log.info("## Reading `airlineStats_OFFLINE` table with filter push down... ##")
