@@ -24,6 +24,7 @@ import java.net.URI;
 import javax.annotation.Nullable;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.Response;
+import org.apache.helix.model.IdealState;
 import org.apache.helix.zookeeper.datamodel.ZNRecord;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadataCustomMapModifier;
@@ -67,10 +68,20 @@ public class ZKOperator {
       long segmentSizeInBytes, boolean enableParallelPushProtection, boolean allowRefresh, HttpHeaders headers)
       throws Exception {
     String segmentName = segmentMetadata.getName();
-    ZNRecord existingSegmentMetadataZNRecord =
-        _pinotHelixResourceManager.getSegmentMetadataZnRecord(tableNameWithType, segmentName);
     boolean refreshOnly =
         Boolean.parseBoolean(headers.getHeaderString(FileUploadDownloadClient.CustomHeaders.REFRESH_ONLY));
+
+    ZNRecord existingSegmentMetadataZNRecord =
+        _pinotHelixResourceManager.getSegmentMetadataZnRecord(tableNameWithType, segmentName);
+    if (existingSegmentMetadataZNRecord != null && shouldProcessAsNewSegment(tableNameWithType, segmentName,
+        existingSegmentMetadataZNRecord, enableParallelPushProtection)) {
+      LOGGER.warn("Removing segment ZK metadata (recovering from previous upload failure) for table: {}, segment: {}",
+          tableNameWithType, segmentName);
+      Preconditions.checkState(_pinotHelixResourceManager.removeSegmentZKMetadata(tableNameWithType, segmentName),
+          "Failed to remove segment ZK metadata for table: %s, segment: %s", tableNameWithType, segmentName);
+      existingSegmentMetadataZNRecord = null;
+    }
+
     if (existingSegmentMetadataZNRecord == null) {
       // Add a new segment
       if (refreshOnly) {
@@ -99,6 +110,44 @@ public class ZKOperator {
     }
   }
 
+  /**
+   * Returns {@code true} when the segment should be processed as new segment.
+   * <p>When segment ZK metadata exists, check if segment exists in the ideal state. If the previous upload failed after
+   * segment ZK metadata is created but before assigning the segment to the ideal state, we want to remove the existing
+   * segment ZK metadata and treat it as a new segment.
+   */
+  private boolean shouldProcessAsNewSegment(String tableNameWithType, String segmentName,
+      ZNRecord existingSegmentMetadataZNRecord, boolean enableParallelPushProtection) {
+    IdealState idealState = _pinotHelixResourceManager.getTableIdealState(tableNameWithType);
+    Preconditions.checkState(idealState != null, "Failed to find ideal state for table: %s", tableNameWithType);
+    if (idealState.getInstanceStateMap(segmentName) != null) {
+      return false;
+    }
+    // Segment does not exist in the ideal state
+    if (enableParallelPushProtection) {
+      // Check segment upload start time when parallel push protection is enabled in case the segment is being uploaded
+      long segmentUploadStartTime = new SegmentZKMetadata(existingSegmentMetadataZNRecord).getSegmentUploadStartTime();
+      if (segmentUploadStartTime > 0) {
+        handleParallelPush(tableNameWithType, segmentName, segmentUploadStartTime);
+      }
+    }
+    return true;
+  }
+
+  private void handleParallelPush(String tableNameWithType, String segmentName, long segmentUploadStartTime) {
+    assert segmentUploadStartTime > 0;
+    if (System.currentTimeMillis() - segmentUploadStartTime > _controllerConf.getSegmentUploadTimeoutInMillis()) {
+      // Last segment upload does not finish properly, replace the segment
+      LOGGER.error("Segment: {} of table: {} was not properly uploaded, replacing it", segmentName, tableNameWithType);
+      _controllerMetrics.addMeteredGlobalValue(ControllerMeter.NUMBER_SEGMENT_UPLOAD_TIMEOUT_EXCEEDED, 1L);
+    } else {
+      // Another segment upload is in progress
+      throw new ControllerApplicationException(LOGGER,
+          String.format("Another segment upload is in progress for segment: %s of table: %s, retry later", segmentName,
+              tableNameWithType), Response.Status.CONFLICT);
+    }
+  }
+
   private void processExistingSegment(String tableNameWithType, SegmentMetadata segmentMetadata,
       FileUploadType uploadType, ZNRecord existingSegmentMetadataZNRecord, @Nullable URI finalSegmentLocationURI,
       File segmentFile, @Nullable String sourceDownloadURIStr, String segmentDownloadURIStr,
@@ -117,17 +166,7 @@ public class ZKOperator {
       // When segment upload start time is larger than 0, that means another upload is in progress
       long segmentUploadStartTime = segmentZKMetadata.getSegmentUploadStartTime();
       if (segmentUploadStartTime > 0) {
-        if (System.currentTimeMillis() - segmentUploadStartTime > _controllerConf.getSegmentUploadTimeoutInMillis()) {
-          // Last segment upload does not finish properly, replace the segment
-          LOGGER.error("Segment: {} of table: {} was not properly uploaded, replacing it", segmentName,
-              tableNameWithType);
-          _controllerMetrics.addMeteredGlobalValue(ControllerMeter.NUMBER_SEGMENT_UPLOAD_TIMEOUT_EXCEEDED, 1L);
-        } else {
-          // Another segment upload is in progress
-          throw new ControllerApplicationException(LOGGER,
-              String.format("Another segment upload is in progress for segment: %s of table: %s, retry later",
-                  segmentName, tableNameWithType), Response.Status.CONFLICT);
-        }
+        handleParallelPush(tableNameWithType, segmentName, segmentUploadStartTime);
       }
 
       // Lock the segment by setting the upload start time in ZK
@@ -258,8 +297,9 @@ public class ZKOperator {
     }
 
     // Lock if enableParallelPushProtection is true.
+    long segmentUploadStartTime = System.currentTimeMillis();
     if (enableParallelPushProtection) {
-      newSegmentZKMetadata.setSegmentUploadStartTime(System.currentTimeMillis());
+      newSegmentZKMetadata.setSegmentUploadStartTime(segmentUploadStartTime);
     }
 
     // Update zk metadata customer map
@@ -284,8 +324,7 @@ public class ZKOperator {
         // Cleanup the Zk entry and the segment from the permanent directory if it exists.
         LOGGER.error("Could not move segment {} from table {} to permanent directory", segmentName, tableNameWithType,
             e);
-        _pinotHelixResourceManager.deleteSegment(tableNameWithType, segmentName);
-        LOGGER.info("Deleted zk entry and segment {} for table {}.", segmentName, tableNameWithType);
+        deleteSegmentIfNeeded(tableNameWithType, segmentName, segmentUploadStartTime, enableParallelPushProtection);
         throw e;
       }
     }
@@ -293,11 +332,11 @@ public class ZKOperator {
     try {
       _pinotHelixResourceManager.assignTableSegment(tableNameWithType, segmentMetadata.getName());
     } catch (Exception e) {
-      // assignTableSegment removes the zk entry. Call deleteSegment to remove the segment from permanent location.
+      // assignTableSegment removes the zk entry.
+      // Call deleteSegment to remove the segment from permanent location if needed.
       LOGGER.error("Caught exception while calling assignTableSegment for adding segment: {} to table: {}", segmentName,
           tableNameWithType, e);
-      _pinotHelixResourceManager.deleteSegment(tableNameWithType, segmentName);
-      LOGGER.info("Deleted zk entry and segment {} for table {}.", segmentName, tableNameWithType);
+      deleteSegmentIfNeeded(tableNameWithType, segmentName, segmentUploadStartTime, enableParallelPushProtection);
       throw e;
     }
 
@@ -305,14 +344,43 @@ public class ZKOperator {
       // Release lock. Expected version will be 0 as we hold a lock and no updates could take place meanwhile.
       newSegmentZKMetadata.setSegmentUploadStartTime(-1);
       if (!_pinotHelixResourceManager.updateZkMetadata(tableNameWithType, newSegmentZKMetadata, 0)) {
-        _pinotHelixResourceManager.deleteSegment(tableNameWithType, segmentName);
-        LOGGER.info("Deleted zk entry and segment {} for table {}.", segmentName, tableNameWithType);
-        throw new RuntimeException(
-            String.format("Failed to update ZK metadata for segment: %s of table: %s", segmentFile, tableNameWithType));
+        // There is a race condition when it took too much time for the 1st segment upload to process (due to slow
+        // PinotFS access), which leads to the 2nd attempt of segment upload, and the 2nd segment upload succeeded.
+        // In this case, when the 1st upload comes back, it shouldn't blindly delete the segment when it failed to
+        // update the zk metadata. Instead, the 1st attempt should validate the upload start time one more time. If the
+        // start time doesn't match with the one persisted in zk metadata, segment deletion should be skipped.
+        String errorMsg =
+            String.format("Failed to update ZK metadata for segment: %s of table: %s", segmentFile, tableNameWithType);
+        LOGGER.error(errorMsg);
+        deleteSegmentIfNeeded(tableNameWithType, segmentName, segmentUploadStartTime, true);
+        throw new RuntimeException(errorMsg);
       }
     }
   }
 
+  /**
+   * Deletes the segment to be uploaded if either one of the criteria is qualified:
+   * 1) the uploadStartTime matches with the one persisted in ZK metadata.
+   * 2) enableParallelPushProtection is not enabled.
+   */
+  private void deleteSegmentIfNeeded(String tableNameWithType, String segmentName, long currentSegmentUploadStartTime,
+      boolean enableParallelPushProtection) {
+    ZNRecord existingSegmentMetadataZNRecord =
+        _pinotHelixResourceManager.getSegmentMetadataZnRecord(tableNameWithType, segmentName);
+    if (existingSegmentMetadataZNRecord == null) {
+      return;
+    }
+    // Check if the upload start time is set by this thread itself, if yes delete the segment.
+    SegmentZKMetadata segmentZKMetadata = new SegmentZKMetadata(existingSegmentMetadataZNRecord);
+    long existingSegmentUploadStartTime = segmentZKMetadata.getSegmentUploadStartTime();
+    LOGGER.info("Parallel push protection is {} for segment: {}.",
+        (enableParallelPushProtection ? "enabled" : "disabled"), segmentName);
+    if (!enableParallelPushProtection || currentSegmentUploadStartTime == existingSegmentUploadStartTime) {
+      _pinotHelixResourceManager.deleteSegment(tableNameWithType, segmentName);
+      LOGGER.info("Deleted zk entry and segment {} for table {}.", segmentName, tableNameWithType);
+    }
+  }
+
   private void copySegmentToDeepStore(String tableNameWithType, String segmentName, FileUploadType uploadType,
       File segmentFile, String sourceDownloadURIStr, URI finalSegmentLocationURI)
       throws Exception {
