@@ -19,6 +19,9 @@
 package org.apache.pinot.core.common;
 
 import com.clearspring.analytics.stream.cardinality.HyperLogLog;
+import com.clearspring.analytics.stream.cardinality.RegisterSet;
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Preconditions;
 import com.google.common.primitives.Longs;
 import com.tdunning.math.stats.MergingDigest;
 import com.tdunning.math.stats.TDigest;
@@ -45,31 +48,32 @@ import it.unimi.dsi.fastutil.longs.LongOpenHashSet;
 import it.unimi.dsi.fastutil.longs.LongSet;
 import it.unimi.dsi.fastutil.objects.ObjectOpenHashSet;
 import it.unimi.dsi.fastutil.objects.ObjectSet;
-import java.io.ByteArrayOutputStream;
-import java.io.DataOutputStream;
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.nio.ByteBuffer;
+import java.nio.IntBuffer;
 import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import javax.annotation.Nullable;
 import org.apache.datasketches.memory.Memory;
 import org.apache.datasketches.theta.Sketch;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.core.query.utils.idset.IdSet;
 import org.apache.pinot.core.query.utils.idset.IdSets;
 import org.apache.pinot.segment.local.customobject.AvgPair;
+import org.apache.pinot.segment.local.customobject.CovarianceTuple;
 import org.apache.pinot.segment.local.customobject.DoubleLongPair;
 import org.apache.pinot.segment.local.customobject.FloatLongPair;
 import org.apache.pinot.segment.local.customobject.IntLongPair;
 import org.apache.pinot.segment.local.customobject.LongLongPair;
 import org.apache.pinot.segment.local.customobject.MinMaxRangePair;
+import org.apache.pinot.segment.local.customobject.PinotFourthMoment;
 import org.apache.pinot.segment.local.customobject.QuantileDigest;
 import org.apache.pinot.segment.local.customobject.StringLongPair;
+import org.apache.pinot.segment.local.customobject.VarianceTuple;
 import org.apache.pinot.segment.local.utils.GeometrySerializer;
 import org.apache.pinot.spi.utils.BigDecimalUtils;
 import org.apache.pinot.spi.utils.ByteArray;
@@ -121,7 +125,10 @@ public class ObjectSerDeUtils {
     FloatLongPair(29),
     DoubleLongPair(30),
     StringLongPair(31),
-    Null(100);
+    CovarianceTuple(32),
+    VarianceTuple(33),
+    PinotFourthMoment(34);
+
     private final int _value;
 
     ObjectType(int value) {
@@ -132,11 +139,7 @@ public class ObjectSerDeUtils {
       return _value;
     }
 
-    public static ObjectType getObjectType(@Nullable Object value) {
-      if (value == null) {
-        return ObjectType.Null;
-      }
-
+    public static ObjectType getObjectType(Object value) {
       if (value instanceof String) {
         return ObjectType.String;
       } else if (value instanceof Long) {
@@ -204,6 +207,12 @@ public class ObjectSerDeUtils {
         return ObjectType.DoubleLongPair;
       } else if (value instanceof StringLongPair) {
         return ObjectType.StringLongPair;
+      } else if (value instanceof CovarianceTuple) {
+        return ObjectType.CovarianceTuple;
+      } else if (value instanceof VarianceTuple) {
+        return ObjectType.VarianceTuple;
+      } else if (value instanceof PinotFourthMoment) {
+        return ObjectType.PinotFourthMoment;
       } else {
         throw new IllegalArgumentException("Unsupported type of value: " + value.getClass().getSimpleName());
       }
@@ -356,8 +365,7 @@ public class ObjectSerDeUtils {
     }
   };
 
-  public static final ObjectSerDe<IntLongPair> INT_LONG_PAIR_SER_DE
-      = new ObjectSerDe<IntLongPair>() {
+  public static final ObjectSerDe<IntLongPair> INT_LONG_PAIR_SER_DE = new ObjectSerDe<IntLongPair>() {
 
     @Override
     public byte[] serialize(IntLongPair intLongPair) {
@@ -375,8 +383,7 @@ public class ObjectSerDeUtils {
     }
   };
 
-  public static final ObjectSerDe<LongLongPair> LONG_LONG_PAIR_SER_DE
-      = new ObjectSerDe<LongLongPair>() {
+  public static final ObjectSerDe<LongLongPair> LONG_LONG_PAIR_SER_DE = new ObjectSerDe<LongLongPair>() {
 
     @Override
     public byte[] serialize(LongLongPair longLongPair) {
@@ -394,8 +401,7 @@ public class ObjectSerDeUtils {
     }
   };
 
-  public static final ObjectSerDe<FloatLongPair> FLOAT_LONG_PAIR_SER_DE
-      = new ObjectSerDe<FloatLongPair>() {
+  public static final ObjectSerDe<FloatLongPair> FLOAT_LONG_PAIR_SER_DE = new ObjectSerDe<FloatLongPair>() {
 
     @Override
     public byte[] serialize(FloatLongPair floatLongPair) {
@@ -412,8 +418,7 @@ public class ObjectSerDeUtils {
       return FloatLongPair.fromByteBuffer(byteBuffer);
     }
   };
-  public static final ObjectSerDe<DoubleLongPair> DOUBLE_LONG_PAIR_SER_DE
-      = new ObjectSerDe<DoubleLongPair>() {
+  public static final ObjectSerDe<DoubleLongPair> DOUBLE_LONG_PAIR_SER_DE = new ObjectSerDe<DoubleLongPair>() {
 
     @Override
     public byte[] serialize(DoubleLongPair doubleLongPair) {
@@ -430,8 +435,7 @@ public class ObjectSerDeUtils {
       return DoubleLongPair.fromByteBuffer(byteBuffer);
     }
   };
-  public static final ObjectSerDe<StringLongPair> STRING_LONG_PAIR_SER_DE
-      = new ObjectSerDe<StringLongPair>() {
+  public static final ObjectSerDe<StringLongPair> STRING_LONG_PAIR_SER_DE = new ObjectSerDe<StringLongPair>() {
 
     @Override
     public byte[] serialize(StringLongPair stringLongPair) {
@@ -449,6 +453,58 @@ public class ObjectSerDeUtils {
     }
   };
 
+  public static final ObjectSerDe<CovarianceTuple> COVARIANCE_TUPLE_OBJECT_SER_DE = new ObjectSerDe<CovarianceTuple>() {
+    @Override
+    public byte[] serialize(CovarianceTuple covarianceTuple) {
+      return covarianceTuple.toBytes();
+    }
+
+    @Override
+    public CovarianceTuple deserialize(byte[] bytes) {
+      return CovarianceTuple.fromBytes(bytes);
+    }
+
+    @Override
+    public CovarianceTuple deserialize(ByteBuffer byteBuffer) {
+      return CovarianceTuple.fromByteBuffer(byteBuffer);
+    }
+  };
+
+  public static final ObjectSerDe<VarianceTuple> VARIANCE_TUPLE_OBJECT_SER_DE = new ObjectSerDe<VarianceTuple>() {
+    @Override
+    public byte[] serialize(VarianceTuple varianceTuple) {
+      return varianceTuple.toBytes();
+    }
+
+    @Override
+    public VarianceTuple deserialize(byte[] bytes) {
+      return VarianceTuple.fromBytes(bytes);
+    }
+
+    @Override
+    public VarianceTuple deserialize(ByteBuffer byteBuffer) {
+      return VarianceTuple.fromByteBuffer(byteBuffer);
+    }
+  };
+
+  public static final ObjectSerDe<PinotFourthMoment> PINOT_FOURTH_MOMENT_OBJECT_SER_DE
+      = new ObjectSerDe<PinotFourthMoment>() {
+    @Override
+    public byte[] serialize(PinotFourthMoment value) {
+      return value.serialize();
+    }
+
+    @Override
+    public PinotFourthMoment deserialize(byte[] bytes) {
+      return PinotFourthMoment.fromBytes(bytes);
+    }
+
+    @Override
+    public PinotFourthMoment deserialize(ByteBuffer byteBuffer) {
+      return PinotFourthMoment.fromBytes(byteBuffer);
+    }
+  };
+
   public static final ObjectSerDe<HyperLogLog> HYPER_LOG_LOG_SER_DE = new ObjectSerDe<HyperLogLog>() {
 
     @Override
@@ -462,21 +518,25 @@ public class ObjectSerDeUtils {
 
     @Override
     public HyperLogLog deserialize(byte[] bytes) {
-      try {
-        return HyperLogLog.Builder.build(bytes);
-      } catch (IOException e) {
-        throw new RuntimeException("Caught exception while de-serializing HyperLogLog", e);
-      }
+      return deserialize(ByteBuffer.wrap(bytes));
     }
 
     @Override
     public HyperLogLog deserialize(ByteBuffer byteBuffer) {
-      byte[] bytes = new byte[byteBuffer.remaining()];
-      byteBuffer.get(bytes);
+      // NOTE: The passed in byte buffer is always BIG ENDIAN
+      return deserialize(byteBuffer.asIntBuffer());
+    }
+
+    private HyperLogLog deserialize(IntBuffer intBuffer) {
       try {
-        return HyperLogLog.Builder.build(bytes);
-      } catch (IOException e) {
-        throw new RuntimeException("Caught exception while de-serializing HyperLogLog", e);
+        // The first 2 integers are constant headers for the HLL: log2m and size in bytes
+        int log2m = intBuffer.get();
+        int numBits = intBuffer.get() >>> 2;
+        int[] bits = new int[numBits];
+        intBuffer.get(bits);
+        return new HyperLogLog(log2m, new RegisterSet(1 << log2m, bits));
+      } catch (RuntimeException e) {
+        throw new RuntimeException("Caught exception while deserializing HyperLogLog", e);
       }
     }
   };
@@ -540,39 +600,39 @@ public class ObjectSerDeUtils {
         return new byte[Integer.BYTES];
       }
 
-      // No need to close these 2 streams
-      ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
-      DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
-
-      try {
-        // Write the size of the map
-        dataOutputStream.writeInt(size);
-
-        // Write the serialized key-value pairs
-        Iterator<Map.Entry<Object, Object>> iterator = map.entrySet().iterator();
-        // First write the key type and value type
-        Map.Entry<Object, Object> firstEntry = iterator.next();
-        Object firstKey = firstEntry.getKey();
-        Object firstValue = firstEntry.getValue();
-        int keyTypeValue = ObjectType.getObjectType(firstKey).getValue();
-        int valueTypeValue = ObjectType.getObjectType(firstValue).getValue();
-        dataOutputStream.writeInt(keyTypeValue);
-        dataOutputStream.writeInt(valueTypeValue);
-        // Then write each key-value pair
-        for (Map.Entry<Object, Object> entry : map.entrySet()) {
-          byte[] keyBytes = ObjectSerDeUtils.serialize(entry.getKey(), keyTypeValue);
-          dataOutputStream.writeInt(keyBytes.length);
-          dataOutputStream.write(keyBytes);
-
-          byte[] valueBytes = ObjectSerDeUtils.serialize(entry.getValue(), valueTypeValue);
-          dataOutputStream.writeInt(valueBytes.length);
-          dataOutputStream.write(valueBytes);
-        }
-      } catch (IOException e) {
-        throw new RuntimeException("Caught exception while serializing Map", e);
+      // Besides the value bytes, we store: size, key type, value type, length for each key, length for each value
+      long bufferSize = (3 + 2 * (long) size) * Integer.BYTES;
+      byte[][] keyBytesArray = new byte[size][];
+      byte[][] valueBytesArray = new byte[size][];
+      Map.Entry<Object, Object> firstEntry = map.entrySet().iterator().next();
+      int keyTypeValue = ObjectType.getObjectType(firstEntry.getKey()).getValue();
+      int valueTypeValue = ObjectType.getObjectType(firstEntry.getValue()).getValue();
+      ObjectSerDe keySerDe = SER_DES[keyTypeValue];
+      ObjectSerDe valueSerDe = SER_DES[valueTypeValue];
+      int index = 0;
+      for (Map.Entry<Object, Object> entry : map.entrySet()) {
+        byte[] keyBytes = keySerDe.serialize(entry.getKey());
+        bufferSize += keyBytes.length;
+        keyBytesArray[index] = keyBytes;
+        byte[] valueBytes = valueSerDe.serialize(entry.getValue());
+        bufferSize += valueBytes.length;
+        valueBytesArray[index++] = valueBytes;
       }
-
-      return byteArrayOutputStream.toByteArray();
+      Preconditions.checkState(bufferSize <= Integer.MAX_VALUE, "Buffer size exceeds 2GB");
+      byte[] bytes = new byte[(int) bufferSize];
+      ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
+      byteBuffer.putInt(size);
+      byteBuffer.putInt(keyTypeValue);
+      byteBuffer.putInt(valueTypeValue);
+      for (int i = 0; i < index; i++) {
+        byte[] keyBytes = keyBytesArray[i];
+        byteBuffer.putInt(keyBytes.length);
+        byteBuffer.put(keyBytes);
+        byte[] valueBytes = valueBytesArray[i];
+        byteBuffer.putInt(valueBytes.length);
+        byteBuffer.put(valueBytes);
+      }
+      return bytes;
     }
 
     @Override
@@ -589,11 +649,11 @@ public class ObjectSerDeUtils {
       }
 
       // De-serialize each key-value pair
-      int keyTypeValue = byteBuffer.getInt();
-      int valueTypeValue = byteBuffer.getInt();
+      ObjectSerDe keySerDe = SER_DES[byteBuffer.getInt()];
+      ObjectSerDe valueSerDe = SER_DES[byteBuffer.getInt()];
       for (int i = 0; i < size; i++) {
-        Object key = ObjectSerDeUtils.deserialize(sliceByteBuffer(byteBuffer, byteBuffer.getInt()), keyTypeValue);
-        Object value = ObjectSerDeUtils.deserialize(sliceByteBuffer(byteBuffer, byteBuffer.getInt()), valueTypeValue);
+        Object key = keySerDe.deserialize(sliceByteBuffer(byteBuffer, byteBuffer.getInt()));
+        Object value = valueSerDe.deserialize(sliceByteBuffer(byteBuffer, byteBuffer.getInt()));
         map.put(key, value);
       }
       return map;
@@ -736,20 +796,24 @@ public class ObjectSerDeUtils {
     @Override
     public byte[] serialize(Set<String> stringSet) {
       int size = stringSet.size();
-      // NOTE: No need to close the ByteArrayOutputStream.
-      ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
-      DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
-      try {
-        dataOutputStream.writeInt(size);
-        for (String value : stringSet) {
-          byte[] bytes = value.getBytes(UTF_8);
-          dataOutputStream.writeInt(bytes.length);
-          dataOutputStream.write(bytes);
-        }
-      } catch (IOException e) {
-        throw new RuntimeException("Caught exception while serializing Set<String>", e);
+      // Besides the value bytes, we store: size, length for each value
+      long bufferSize = (1 + (long) size) * Integer.BYTES;
+      byte[][] valueBytesArray = new byte[size][];
+      int index = 0;
+      for (String value : stringSet) {
+        byte[] valueBytes = value.getBytes(UTF_8);
+        bufferSize += valueBytes.length;
+        valueBytesArray[index++] = valueBytes;
       }
-      return byteArrayOutputStream.toByteArray();
+      Preconditions.checkState(bufferSize <= Integer.MAX_VALUE, "Buffer size exceeds 2GB");
+      byte[] bytes = new byte[(int) bufferSize];
+      ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
+      byteBuffer.putInt(size);
+      for (byte[] valueBytes : valueBytesArray) {
+        byteBuffer.putInt(valueBytes.length);
+        byteBuffer.put(valueBytes);
+      }
+      return bytes;
     }
 
     @Override
@@ -776,20 +840,21 @@ public class ObjectSerDeUtils {
     @Override
     public byte[] serialize(Set<ByteArray> bytesSet) {
       int size = bytesSet.size();
-      // NOTE: No need to close the ByteArrayOutputStream.
-      ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
-      DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
-      try {
-        dataOutputStream.writeInt(size);
-        for (ByteArray value : bytesSet) {
-          byte[] bytes = value.getBytes();
-          dataOutputStream.writeInt(bytes.length);
-          dataOutputStream.write(bytes);
-        }
-      } catch (IOException e) {
-        throw new RuntimeException("Caught exception while serializing Set<ByteArray>", e);
+      // Besides the value bytes, we store: size, length for each value
+      long bufferSize = (1 + (long) size) * Integer.BYTES;
+      for (ByteArray value : bytesSet) {
+        bufferSize += value.length();
       }
-      return byteArrayOutputStream.toByteArray();
+      Preconditions.checkState(bufferSize <= Integer.MAX_VALUE, "Buffer size exceeds 2GB");
+      byte[] bytes = new byte[(int) bufferSize];
+      ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
+      byteBuffer.putInt(size);
+      for (ByteArray value : bytesSet) {
+        byte[] valueBytes = value.getBytes();
+        byteBuffer.putInt(valueBytes.length);
+        byteBuffer.put(valueBytes);
+      }
+      return bytes;
     }
 
     @Override
@@ -941,30 +1006,27 @@ public class ObjectSerDeUtils {
         return new byte[Integer.BYTES];
       }
 
-      // No need to close these 2 streams (close() is no-op)
-      ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
-      DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
-
-      try {
-        // Write the size of the list
-        dataOutputStream.writeInt(size);
-
-        // Write the value type
-        Object firstValue = list.get(0);
-        int valueType = ObjectType.getObjectType(firstValue).getValue();
-        dataOutputStream.writeInt(valueType);
-
-        // Write the serialized values
-        for (Object value : list) {
-          byte[] bytes = ObjectSerDeUtils.serialize(value, valueType);
-          dataOutputStream.writeInt(bytes.length);
-          dataOutputStream.write(bytes);
-        }
-      } catch (IOException e) {
-        throw new RuntimeException("Caught exception while serializing List", e);
+      // Besides the value bytes, we store: size, value type, length for each value
+      long bufferSize = (2 + (long) size) * Integer.BYTES;
+      byte[][] valueBytesArray = new byte[size][];
+      int valueType = ObjectType.getObjectType(list.get(0)).getValue();
+      ObjectSerDe serDe = SER_DES[valueType];
+      int index = 0;
+      for (Object value : list) {
+        byte[] valueBytes = serDe.serialize(value);
+        bufferSize += valueBytes.length;
+        valueBytesArray[index++] = valueBytes;
       }
-
-      return byteArrayOutputStream.toByteArray();
+      Preconditions.checkState(bufferSize <= Integer.MAX_VALUE, "Buffer size exceeds 2GB");
+      byte[] bytes = new byte[(int) bufferSize];
+      ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
+      byteBuffer.putInt(size);
+      byteBuffer.putInt(valueType);
+      for (byte[] valueBytes : valueBytesArray) {
+        byteBuffer.putInt(valueBytes.length);
+        byteBuffer.put(valueBytes);
+      }
+      return bytes;
     }
 
     @Override
@@ -979,12 +1041,12 @@ public class ObjectSerDeUtils {
 
       // De-serialize the values
       if (size != 0) {
-        int valueType = byteBuffer.getInt();
+        ObjectSerDe serDe = SER_DES[byteBuffer.getInt()];
         for (int i = 0; i < size; i++) {
           int numBytes = byteBuffer.getInt();
           ByteBuffer slice = byteBuffer.slice();
           slice.limit(numBytes);
-          list.add(ObjectSerDeUtils.deserialize(slice, valueType));
+          list.add(serDe.deserialize(slice));
           byteBuffer.position(byteBuffer.position() + numBytes);
         }
       }
@@ -1171,35 +1233,28 @@ public class ObjectSerDeUtils {
       LONG_LONG_PAIR_SER_DE,
       FLOAT_LONG_PAIR_SER_DE,
       DOUBLE_LONG_PAIR_SER_DE,
-      STRING_LONG_PAIR_SER_DE
+      STRING_LONG_PAIR_SER_DE,
+      COVARIANCE_TUPLE_OBJECT_SER_DE,
+      VARIANCE_TUPLE_OBJECT_SER_DE,
+      PINOT_FOURTH_MOMENT_OBJECT_SER_DE
   };
   //@formatter:on
 
-  public static byte[] serialize(Object value) {
-    return serialize(value, ObjectType.getObjectType(value)._value);
-  }
-
-  public static byte[] serialize(Object value, ObjectType objectType) {
-    return serialize(value, objectType._value);
-  }
-
   public static byte[] serialize(Object value, int objectTypeValue) {
     return SER_DES[objectTypeValue].serialize(value);
   }
 
+  public static <T> T deserialize(DataTable.CustomObject customObject) {
+    return (T) SER_DES[customObject.getType()].deserialize(customObject.getBuffer());
+  }
+
+  @VisibleForTesting
+  public static byte[] serialize(Object value) {
+    return serialize(value, ObjectType.getObjectType(value)._value);
+  }
+
+  @VisibleForTesting
   public static <T> T deserialize(byte[] bytes, ObjectType objectType) {
-    return deserialize(bytes, objectType._value);
-  }
-
-  public static <T> T deserialize(byte[] bytes, int objectTypeValue) {
-    return (T) SER_DES[objectTypeValue].deserialize(bytes);
-  }
-
-  public static <T> T deserialize(ByteBuffer byteBuffer, ObjectType objectType) {
-    return deserialize(byteBuffer, objectType._value);
-  }
-
-  public static <T> T deserialize(ByteBuffer byteBuffer, int objectTypeValue) {
-    return (T) SER_DES[objectTypeValue].deserialize(byteBuffer);
+    return (T) SER_DES[objectType._value].deserialize(bytes);
   }
 }
