@@ -32,7 +32,11 @@ public abstract class BaseOperator<T extends Block> implements Operator<T> {
 
   @Override
   public final T nextBlock() {
-    if (Thread.interrupted()) {
+    /* Worker also checks its corresponding runner thread's interruption periodically, worker will abort if it finds
+       runner's flag is raised. If the runner thread has already acted upon the flag and reset it, then the runner
+       itself will cancel all worker's futures. Therefore, the worker will interrupt even if we only kill the runner
+       thread. */
+    if (Tracing.ThreadAccountantOps.isInterrupted()) {
       throw new EarlyTerminationException();
     }
     try (InvocationScope ignored = Tracing.getTracer().createScope(getClass())) {
