@@ -18,11 +18,10 @@
  */
 package org.apache.pinot.core.operator.combine;
 
-import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.ExecutorService;
 import org.apache.pinot.core.common.Operator;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.DistinctResultsBlock;
 import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.core.query.request.context.QueryContext;
 
@@ -31,7 +30,7 @@ import org.apache.pinot.core.query.request.context.QueryContext;
  * Combine operator for distinct queries.
  */
 @SuppressWarnings("rawtypes")
-public class DistinctCombineOperator extends BaseCombineOperator {
+public class DistinctCombineOperator extends BaseCombineOperator<DistinctResultsBlock> {
   private static final String EXPLAIN_NAME = "COMBINE_DISTINCT";
 
   private final boolean _hasOrderBy;
@@ -41,40 +40,32 @@ public class DistinctCombineOperator extends BaseCombineOperator {
     _hasOrderBy = queryContext.getOrderByExpressions() != null;
   }
 
-
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
   }
 
   @Override
-  protected boolean isQuerySatisfied(IntermediateResultsBlock resultsBlock) {
+  protected boolean isQuerySatisfied(DistinctResultsBlock resultsBlock) {
     if (_hasOrderBy) {
       return false;
     }
-    List<Object> result = resultsBlock.getAggregationResult();
-    assert result != null && result.size() == 1 && result.get(0) instanceof DistinctTable;
-    DistinctTable distinctTable = (DistinctTable) result.get(0);
-    return distinctTable.size() >= _queryContext.getLimit();
+    return resultsBlock.getDistinctTable().size() >= _queryContext.getLimit();
   }
 
   @Override
-  protected void mergeResultsBlocks(IntermediateResultsBlock mergedBlock, IntermediateResultsBlock blockToMerge) {
-    // TODO: Use a separate way to represent DISTINCT instead of aggregation.
-    List<Object> mergedResults = mergedBlock.getAggregationResult();
-    assert mergedResults != null && mergedResults.size() == 1 && mergedResults.get(0) instanceof DistinctTable;
-    DistinctTable mergedDistinctTable = (DistinctTable) mergedResults.get(0);
-
-    List<Object> resultsToMerge = blockToMerge.getAggregationResult();
-    assert resultsToMerge != null && resultsToMerge.size() == 1 && resultsToMerge.get(0) instanceof DistinctTable;
-    DistinctTable distinctTableToMerge = (DistinctTable) resultsToMerge.get(0);
+  protected void mergeResultsBlocks(DistinctResultsBlock mergedBlock, DistinctResultsBlock blockToMerge) {
+    DistinctTable mergedDistinctTable = mergedBlock.getDistinctTable();
+    DistinctTable distinctTableToMerge = blockToMerge.getDistinctTable();
+    assert mergedDistinctTable != null && distinctTableToMerge != null;
 
     // Convert the merged table into a main table if necessary in order to merge other tables
     if (!mergedDistinctTable.isMainTable()) {
-      DistinctTable mainDistinctTable = new DistinctTable(distinctTableToMerge.getDataSchema(),
-          _queryContext.getOrderByExpressions(), _queryContext.getLimit(), _queryContext.isNullHandlingEnabled());
+      DistinctTable mainDistinctTable =
+          new DistinctTable(distinctTableToMerge.getDataSchema(), _queryContext.getOrderByExpressions(),
+              _queryContext.getLimit(), _queryContext.isNullHandlingEnabled());
       mainDistinctTable.mergeTable(mergedDistinctTable);
-      mergedBlock.setAggregationResults(Collections.singletonList(mainDistinctTable));
+      mergedBlock.setDistinctTable(mainDistinctTable);
       mergedDistinctTable = mainDistinctTable;
     }
 
