@@ -24,7 +24,8 @@ import java.util.concurrent.ExecutorService;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.core.common.Operator;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.BaseResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.query.selection.SelectionOperatorUtils;
 import org.slf4j.Logger;
@@ -38,7 +39,7 @@ import org.slf4j.LoggerFactory;
  * <p>NOTE: Selection order-by query with LIMIT 0 is treated as selection only query.
  */
 @SuppressWarnings("rawtypes")
-public class SelectionOnlyCombineOperator extends BaseCombineOperator {
+public class SelectionOnlyCombineOperator extends BaseCombineOperator<SelectionResultsBlock> {
   private static final Logger LOGGER = LoggerFactory.getLogger(SelectionOnlyCombineOperator.class);
 
   private static final String EXPLAIN_NAME = "COMBINE_SELECT";
@@ -51,17 +52,16 @@ public class SelectionOnlyCombineOperator extends BaseCombineOperator {
     _numRowsToKeep = queryContext.getLimit();
   }
 
-
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
   }
 
   @Override
-  protected IntermediateResultsBlock getNextBlock() {
+  protected BaseResultsBlock getNextBlock() {
     // For LIMIT 0 query, only process one segment to get the data schema
     if (_numRowsToKeep == 0) {
-      IntermediateResultsBlock resultsBlock = (IntermediateResultsBlock) _operators.get(0).nextBlock();
+      BaseResultsBlock resultsBlock = (BaseResultsBlock) _operators.get(0).nextBlock();
       CombineOperatorUtils.setExecutionStatistics(resultsBlock, _operators, 0, 1);
       return resultsBlock;
     }
@@ -70,14 +70,12 @@ public class SelectionOnlyCombineOperator extends BaseCombineOperator {
   }
 
   @Override
-  protected boolean isQuerySatisfied(IntermediateResultsBlock resultsBlock) {
-    Collection<Object[]> selectionResult = resultsBlock.getSelectionResult();
-    assert selectionResult != null;
-    return selectionResult.size() == _numRowsToKeep;
+  protected boolean isQuerySatisfied(SelectionResultsBlock resultsBlock) {
+    return resultsBlock.getRows().size() == _numRowsToKeep;
   }
 
   @Override
-  protected void mergeResultsBlocks(IntermediateResultsBlock mergedBlock, IntermediateResultsBlock blockToMerge) {
+  protected void mergeResultsBlocks(SelectionResultsBlock mergedBlock, SelectionResultsBlock blockToMerge) {
     DataSchema mergedDataSchema = mergedBlock.getDataSchema();
     DataSchema dataSchemaToMerge = blockToMerge.getDataSchema();
     assert mergedDataSchema != null && dataSchemaToMerge != null;
@@ -92,8 +90,8 @@ public class SelectionOnlyCombineOperator extends BaseCombineOperator {
       return;
     }
 
-    Collection<Object[]> mergedRows = mergedBlock.getSelectionResult();
-    Collection<Object[]> rowsToMerge = blockToMerge.getSelectionResult();
+    Collection<Object[]> mergedRows = mergedBlock.getRows();
+    Collection<Object[]> rowsToMerge = blockToMerge.getRows();
     assert mergedRows != null && rowsToMerge != null;
     SelectionOperatorUtils.mergeWithoutOrdering(mergedRows, rowsToMerge, _numRowsToKeep);
   }
