@@ -218,6 +218,14 @@ public class TransformBlockValSet implements BlockValSet {
     }
   }
 
+  @Override
+  public byte[][][] getBytesValuesMV() {
+    try (InvocationScope scope = Tracing.getTracer().createScope(TransformBlockValSet.class)) {
+      recordTransformValues(scope, DataType.BYTES, false);
+      return _transformFunction.transformToBytesValuesMV(_projectionBlock);
+    }
+  }
+
   @Override
   public int[] getNumMVEntries() {
     if (_numMVEntries == null) {
