@@ -20,6 +20,7 @@ package org.apache.pinot.core.operator.filter;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Map;
 import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.operator.blocks.FilterBlock;
 import org.apache.pinot.core.operator.docidsets.AndDocIdSet;
@@ -33,9 +34,15 @@ public class AndFilterOperator extends BaseFilterOperator {
   private static final String EXPLAIN_NAME = "FILTER_AND";
 
   private final List<BaseFilterOperator> _filterOperators;
+  private final Map<String, String> _queryOptions;
+
+  public AndFilterOperator(List<BaseFilterOperator> filterOperators, Map<String, String> queryOptions) {
+    _filterOperators = filterOperators;
+    _queryOptions = queryOptions;
+  }
 
   public AndFilterOperator(List<BaseFilterOperator> filterOperators) {
-    _filterOperators = filterOperators;
+    this(filterOperators, null);
   }
 
   @Override
@@ -45,7 +52,7 @@ public class AndFilterOperator extends BaseFilterOperator {
     for (BaseFilterOperator filterOperator : _filterOperators) {
       filterBlockDocIdSets.add(filterOperator.nextBlock().getBlockDocIdSet());
     }
-    return new FilterBlock(new AndDocIdSet(filterBlockDocIdSets));
+    return new FilterBlock(new AndDocIdSet(filterBlockDocIdSets, _queryOptions));
   }
 
   @Override
