@@ -101,6 +101,11 @@ public class EqualsPredicateEvaluatorFactory {
       }
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public boolean applySV(int dictId) {
       return _matchingDictId == dictId;
@@ -133,6 +138,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.INT;
@@ -165,6 +175,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.LONG;
@@ -197,6 +212,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.FLOAT;
@@ -229,6 +249,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.DOUBLE;
@@ -261,6 +286,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BIG_DECIMAL;
@@ -280,6 +310,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.STRING;
@@ -299,6 +334,11 @@ public class EqualsPredicateEvaluatorFactory {
       _matchingValue = matchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return 1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BYTES;
