@@ -176,6 +176,11 @@ public class InPredicateEvaluatorFactory {
       return _numMatchingDictIds;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return getNumMatchingDictIds();
+    }
+
     @Override
     public int[] getMatchingDictIds() {
       if (_matchingDictIds == null) {
@@ -206,6 +211,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.INT;
@@ -238,6 +248,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.LONG;
@@ -270,6 +285,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.FLOAT;
@@ -302,6 +322,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.DOUBLE;
@@ -340,6 +365,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BIG_DECIMAL;
@@ -359,6 +389,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.STRING;
@@ -378,6 +413,11 @@ public class InPredicateEvaluatorFactory {
       _matchingValues = matchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BYTES;
