@@ -169,6 +169,11 @@ public class NotInPredicateEvaluatorFactory {
       _dictionary = dictionary;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_numNonMatchingDictIds;
+    }
+
     @Override
     public boolean applySV(int dictId) {
       return !_nonMatchingDictIdSet.contains(dictId);
@@ -224,6 +229,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.INT;
@@ -256,6 +266,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.LONG;
@@ -288,6 +303,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.FLOAT;
@@ -320,6 +340,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.DOUBLE;
@@ -355,6 +380,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BIG_DECIMAL;
@@ -374,6 +404,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.STRING;
@@ -393,6 +428,11 @@ public class NotInPredicateEvaluatorFactory {
       _nonMatchingValues = nonMatchingValues;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -_nonMatchingValues.size();
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BYTES;
