@@ -28,8 +28,7 @@ import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.data.table.Record;
 import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.core.operator.ExecutionStatistics;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
-import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
+import org.apache.pinot.core.operator.blocks.results.DistinctResultsBlock;
 import org.apache.pinot.core.query.aggregation.function.DistinctAggregationFunction;
 import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
@@ -40,7 +39,7 @@ import org.apache.pinot.spi.data.FieldSpec;
 /**
  * Operator which executes DISTINCT operation based on dictionary
  */
-public class DictionaryBasedDistinctOperator extends BaseOperator<IntermediateResultsBlock> {
+public class DictionaryBasedDistinctOperator extends BaseOperator<DistinctResultsBlock> {
   private static final String EXPLAIN_NAME = "DISTINCT_DICTIONARY";
 
   private final DistinctAggregationFunction _distinctAggregationFunction;
@@ -56,7 +55,6 @@ public class DictionaryBasedDistinctOperator extends BaseOperator<IntermediateRe
   public DictionaryBasedDistinctOperator(FieldSpec.DataType dataType,
       DistinctAggregationFunction distinctAggregationFunction, Dictionary dictionary, int numTotalDocs,
       boolean nullHandlingEnabled) {
-
     _dataType = dataType;
     _distinctAggregationFunction = distinctAggregationFunction;
     _dictionary = dictionary;
@@ -64,20 +62,16 @@ public class DictionaryBasedDistinctOperator extends BaseOperator<IntermediateRe
     _nullHandlingEnabled = nullHandlingEnabled;
 
     List<OrderByExpressionContext> orderByExpressionContexts = _distinctAggregationFunction.getOrderByExpressions();
-
     if (orderByExpressionContexts != null) {
       OrderByExpressionContext orderByExpressionContext = orderByExpressionContexts.get(0);
-
       _isAscending = orderByExpressionContext.isAsc();
       _hasOrderBy = true;
     }
   }
 
   @Override
-  protected IntermediateResultsBlock getNextBlock() {
-    DistinctTable distinctTable = buildResult();
-    return new IntermediateResultsBlock(new AggregationFunction[]{_distinctAggregationFunction},
-        Collections.singletonList(distinctTable), _nullHandlingEnabled);
+  protected DistinctResultsBlock getNextBlock() {
+    return new DistinctResultsBlock(_distinctAggregationFunction, buildResult());
   }
 
   /**
@@ -124,8 +118,9 @@ public class DictionaryBasedDistinctOperator extends BaseOperator<IntermediateRe
         }
       } else {
         // DictionaryBasedDistinctOperator cannot handle nulls.
-        DistinctTable distinctTable = new DistinctTable(
-            dataSchema, _distinctAggregationFunction.getOrderByExpressions(), limit, _nullHandlingEnabled);
+        DistinctTable distinctTable =
+            new DistinctTable(dataSchema, _distinctAggregationFunction.getOrderByExpressions(), limit,
+                _nullHandlingEnabled);
 
         _numDocsScanned = dictLength;
         for (int i = 0; i < dictLength; i++) {
