@@ -23,14 +23,12 @@ import java.util.List;
 import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.core.operator.ExecutionStatistics;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
 import org.apache.pinot.core.operator.blocks.TransformBlock;
+import org.apache.pinot.core.operator.blocks.results.DistinctResultsBlock;
 import org.apache.pinot.core.operator.transform.TransformOperator;
-import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
 import org.apache.pinot.core.query.aggregation.function.DistinctAggregationFunction;
 import org.apache.pinot.core.query.distinct.DistinctExecutor;
 import org.apache.pinot.core.query.distinct.DistinctExecutorFactory;
-import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.segment.spi.IndexSegment;
 
@@ -38,14 +36,13 @@ import org.apache.pinot.segment.spi.IndexSegment;
 /**
  * Operator for distinct queries on a single segment.
  */
-public class DistinctOperator extends BaseOperator<IntermediateResultsBlock> {
+public class DistinctOperator extends BaseOperator<DistinctResultsBlock> {
   private static final String EXPLAIN_NAME = "DISTINCT";
 
   private final IndexSegment _indexSegment;
   private final DistinctAggregationFunction _distinctAggregationFunction;
   private final TransformOperator _transformOperator;
   private final DistinctExecutor _distinctExecutor;
-  private final QueryContext _queryContext;
 
   private int _numDocsScanned = 0;
 
@@ -54,13 +51,12 @@ public class DistinctOperator extends BaseOperator<IntermediateResultsBlock> {
     _indexSegment = indexSegment;
     _distinctAggregationFunction = distinctAggregationFunction;
     _transformOperator = transformOperator;
-    _queryContext = queryContext;
-    _distinctExecutor = DistinctExecutorFactory.getDistinctExecutor(
-        distinctAggregationFunction, transformOperator, _queryContext.isNullHandlingEnabled());
+    _distinctExecutor = DistinctExecutorFactory.getDistinctExecutor(distinctAggregationFunction, transformOperator,
+        queryContext.isNullHandlingEnabled());
   }
 
   @Override
-  protected IntermediateResultsBlock getNextBlock() {
+  protected DistinctResultsBlock getNextBlock() {
     TransformBlock transformBlock;
     while ((transformBlock = _transformOperator.nextBlock()) != null) {
       _numDocsScanned += transformBlock.getNumDocs();
@@ -68,10 +64,7 @@ public class DistinctOperator extends BaseOperator<IntermediateResultsBlock> {
         break;
       }
     }
-    DistinctTable distinctTable = _distinctExecutor.getResult();
-    // TODO: Use a separate way to represent DISTINCT instead of aggregation.
-    return new IntermediateResultsBlock(new AggregationFunction[]{_distinctAggregationFunction},
-        Collections.singletonList(distinctTable), _queryContext.isNullHandlingEnabled());
+    return new DistinctResultsBlock(_distinctAggregationFunction, _distinctExecutor.getResult());
   }
 
   @Override
@@ -79,6 +72,7 @@ public class DistinctOperator extends BaseOperator<IntermediateResultsBlock> {
     return Collections.singletonList(_transformOperator);
   }
 
+  @Override
   public IndexSegment getIndexSegment() {
     return _indexSegment;
   }
