@@ -20,19 +20,21 @@ package org.apache.pinot.core.operator.streaming;
 
 import io.grpc.stub.StreamObserver;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicLong;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.proto.Server;
 import org.apache.pinot.common.utils.DataSchema;
-import org.apache.pinot.common.utils.DataTable;
 import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.operator.AcquireReleaseColumnsSegmentOperator;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.BaseResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.ExceptionResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.MetadataResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
 import org.apache.pinot.core.operator.combine.BaseCombineOperator;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.query.selection.SelectionOperatorUtils;
@@ -43,16 +45,14 @@ import org.slf4j.LoggerFactory;
 /**
  * Combine operator for selection only streaming queries.
  */
-@SuppressWarnings({"rawtypes", "unchecked"})
-public class StreamingSelectionOnlyCombineOperator extends BaseCombineOperator {
+@SuppressWarnings("rawtypes")
+public class StreamingSelectionOnlyCombineOperator extends BaseCombineOperator<SelectionResultsBlock> {
   private static final Logger LOGGER = LoggerFactory.getLogger(StreamingSelectionOnlyCombineOperator.class);
 
   private static final String EXPLAIN_NAME = "SELECT_STREAMING_COMBINE";
 
-  // Special IntermediateResultsBlock to indicate that this is the last results block for an operator
-  private static final IntermediateResultsBlock LAST_RESULTS_BLOCK =
-      new IntermediateResultsBlock(new DataSchema(new String[0], new DataSchema.ColumnDataType[0]),
-          Collections.emptyList(), false);
+  // Special results block to indicate that this is the last results block for an operator
+  private static final MetadataResultsBlock LAST_RESULTS_BLOCK = new MetadataResultsBlock();
 
   private final StreamObserver<Server.ServerResponse> _streamObserver;
   private final int _limit;
@@ -65,23 +65,23 @@ public class StreamingSelectionOnlyCombineOperator extends BaseCombineOperator {
     _limit = queryContext.getLimit();
   }
 
-
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
   }
 
   @Override
-  protected void processSegments(int threadIndex) {
-    for (int operatorIndex = threadIndex; operatorIndex < _numOperators; operatorIndex += _numTasks) {
-      Operator<IntermediateResultsBlock> operator = _operators.get(operatorIndex);
-      IntermediateResultsBlock resultsBlock;
+  protected void processSegments() {
+    int operatorId;
+    while ((operatorId = _nextOperatorId.getAndIncrement()) < _numOperators) {
+      Operator operator = _operators.get(operatorId);
+      SelectionResultsBlock resultsBlock;
       try {
         if (operator instanceof AcquireReleaseColumnsSegmentOperator) {
           ((AcquireReleaseColumnsSegmentOperator) operator).acquire();
         }
-        while ((resultsBlock = operator.nextBlock()) != null) {
-          Collection<Object[]> rows = resultsBlock.getSelectionResult();
+        while ((resultsBlock = (SelectionResultsBlock) operator.nextBlock()) != null) {
+          Collection<Object[]> rows = resultsBlock.getRows();
           assert rows != null;
           long numRowsCollected = _numRowsCollected.addAndGet(rows.size());
           _blockingQueue.offer(resultsBlock);
@@ -99,18 +99,18 @@ public class StreamingSelectionOnlyCombineOperator extends BaseCombineOperator {
   }
 
   @Override
-  protected IntermediateResultsBlock mergeResults()
+  protected BaseResultsBlock mergeResults()
       throws Exception {
     long numRowsCollected = 0;
     int numOperatorsFinished = 0;
     long endTimeMs = _queryContext.getEndTimeMs();
     while (numRowsCollected < _limit && numOperatorsFinished < _numOperators) {
-      IntermediateResultsBlock resultsBlock =
+      BaseResultsBlock resultsBlock =
           _blockingQueue.poll(endTimeMs - System.currentTimeMillis(), TimeUnit.MILLISECONDS);
       if (resultsBlock == null) {
         // Query times out, skip streaming the remaining results blocks
         LOGGER.error("Timed out while polling results block (query: {})", _queryContext);
-        return new IntermediateResultsBlock(QueryException.getException(QueryException.EXECUTION_TIMEOUT_ERROR,
+        return new ExceptionResultsBlock(QueryException.getException(QueryException.EXECUTION_TIMEOUT_ERROR,
             new TimeoutException("Timed out while polling results block")));
       }
       if (resultsBlock.getProcessingExceptions() != null) {
@@ -122,19 +122,20 @@ public class StreamingSelectionOnlyCombineOperator extends BaseCombineOperator {
         numOperatorsFinished++;
         continue;
       }
-      DataSchema dataSchema = resultsBlock.getDataSchema();
-      Collection<Object[]> rows = resultsBlock.getSelectionResult();
+      SelectionResultsBlock selectionResultsBlock = (SelectionResultsBlock) resultsBlock;
+      DataSchema dataSchema = selectionResultsBlock.getDataSchema();
+      Collection<Object[]> rows = selectionResultsBlock.getRows();
       assert dataSchema != null && rows != null;
       numRowsCollected += rows.size();
-      DataTable dataTable = SelectionOperatorUtils.getDataTableFromRows(
-          rows, dataSchema, _queryContext.isNullHandlingEnabled());
+      DataTable dataTable =
+          SelectionOperatorUtils.getDataTableFromRows(rows, dataSchema, _queryContext.isNullHandlingEnabled());
       _streamObserver.onNext(StreamingResponseUtils.getDataResponse(dataTable));
     }
-    // Return an empty results block for the metadata
-    return new IntermediateResultsBlock();
+    // Return a metadata results block in the end
+    return new MetadataResultsBlock();
   }
 
   @Override
-  protected void mergeResultsBlocks(IntermediateResultsBlock mergedBlock, IntermediateResultsBlock blockToMerge) {
+  protected void mergeResultsBlocks(SelectionResultsBlock mergedBlock, SelectionResultsBlock blockToMerge) {
   }
 }
