@@ -38,11 +38,6 @@ import org.apache.pinot.spi.data.FieldSpec.DataType;
 public class ArrayMinTransformFunction extends BaseTransformFunction {
   public static final String FUNCTION_NAME = "arrayMin";
 
-  private int[] _intValuesSV;
-  private long[] _longValuesSV;
-  private float[] _floatValuesSV;
-  private double[] _doubleValuesSV;
-  private String[] _stringValuesSV;
   private TransformFunction _argument;
   private TransformResultMetadata _resultMetadata;
 
@@ -78,10 +73,8 @@ public class ArrayMinTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.INT) {
       return super.transformToIntValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_intValuesSV == null || _intValuesSV.length < length) {
+    if (_intValuesSV == null) {
       _intValuesSV = new int[length];
     }
     int[][] intValuesMV = _argument.transformToIntValuesMV(projectionBlock);
@@ -100,13 +93,10 @@ public class ArrayMinTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.LONG) {
       return super.transformToLongValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_longValuesSV == null || _longValuesSV.length < length) {
+    if (_longValuesSV == null) {
       _longValuesSV = new long[length];
     }
-
     long[][] longValuesMV = _argument.transformToLongValuesMV(projectionBlock);
     for (int i = 0; i < length; i++) {
       long minRes = Long.MAX_VALUE;
@@ -123,13 +113,10 @@ public class ArrayMinTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.FLOAT) {
       return super.transformToFloatValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_floatValuesSV == null || _floatValuesSV.length < length) {
+    if (_floatValuesSV == null) {
       _floatValuesSV = new float[length];
     }
-
     float[][] floatValuesMV = _argument.transformToFloatValuesMV(projectionBlock);
     for (int i = 0; i < length; i++) {
       float minRes = Float.POSITIVE_INFINITY;
@@ -146,13 +133,10 @@ public class ArrayMinTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.DOUBLE) {
       return super.transformToDoubleValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_doubleValuesSV == null || _doubleValuesSV.length < length) {
+    if (_doubleValuesSV == null) {
       _doubleValuesSV = new double[length];
     }
-
     double[][] doubleValuesMV = _argument.transformToDoubleValuesMV(projectionBlock);
     for (int i = 0; i < length; i++) {
       double minRes = Double.POSITIVE_INFINITY;
@@ -169,13 +153,10 @@ public class ArrayMinTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.STRING) {
       return super.transformToStringValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_stringValuesSV == null || _stringValuesSV.length < length) {
+    if (_stringValuesSV == null) {
       _stringValuesSV = new String[length];
     }
-
     String[][] stringValuesMV = _argument.transformToStringValuesMV(projectionBlock);
     for (int i = 0; i < length; i++) {
       String minRes = null;
