@@ -141,6 +141,11 @@ public class IdentifierTransformFunction implements TransformFunction, PushDownT
     return projectionBlock.getBlockValueSet(_columnName).getStringValuesMV();
   }
 
+  @Override
+  public byte[][][] transformToBytesValuesMV(ProjectionBlock projectionBlock) {
+    return projectionBlock.getBlockValueSet(_columnName).getBytesValuesMV();
+  }
+
   @Override
   public void transformToIntValuesSV(ProjectionBlock projectionBlock, TransformEvaluator evaluator, int[] buffer) {
     projectionBlock.fillValues(_columnName, evaluator, buffer);
