@@ -24,7 +24,6 @@ import java.util.Map;
 import org.apache.pinot.core.operator.blocks.ProjectionBlock;
 import org.apache.pinot.core.operator.transform.TransformResultMetadata;
 import org.apache.pinot.segment.spi.datasource.DataSource;
-import org.apache.pinot.spi.utils.ArrayCopyUtils;
 
 
 /**
@@ -33,15 +32,15 @@ import org.apache.pinot.spi.utils.ArrayCopyUtils;
  */
 public abstract class LogicalOperatorTransformFunction extends BaseTransformFunction {
   protected List<TransformFunction> _arguments;
-  protected int[] _results;
 
   @Override
   public void init(List<TransformFunction> arguments, Map<String, DataSource> dataSourceMap) {
     _arguments = arguments;
     int numArguments = arguments.size();
     if (numArguments <= 1) {
-      throw new IllegalArgumentException("Expect more than 1 argument for logical operator [" + getName() + "], args ["
-          + Arrays.toString(arguments.toArray()) + "].");
+      throw new IllegalArgumentException(
+          "Expect more than 1 argument for logical operator [" + getName() + "], args [" + Arrays.toString(
+              arguments.toArray()) + "].");
     }
     for (int i = 0; i < numArguments; i++) {
       TransformResultMetadata argumentMetadata = arguments.get(i).getResultMetadata();
@@ -60,20 +59,19 @@ public abstract class LogicalOperatorTransformFunction extends BaseTransformFunc
   @Override
   public int[] transformToIntValuesSV(ProjectionBlock projectionBlock) {
     int numDocs = projectionBlock.getNumDocs();
-
-    if (_results == null || _results.length < numDocs) {
-      _results = new int[numDocs];
+    if (_intValuesSV == null) {
+      _intValuesSV = new int[numDocs];
     }
-    ArrayCopyUtils.copy(_arguments.get(0).transformToIntValuesSV(projectionBlock), _results, numDocs);
+    System.arraycopy(_arguments.get(0).transformToIntValuesSV(projectionBlock), 0, _intValuesSV, 0, numDocs);
     int numArguments = _arguments.size();
     for (int i = 1; i < numArguments; i++) {
       TransformFunction transformFunction = _arguments.get(i);
       int[] results = transformFunction.transformToIntValuesSV(projectionBlock);
       for (int j = 0; j < numDocs; j++) {
-        _results[j] = getLogicalFuncResult(_results[j], results[j]);
+        _intValuesSV[j] = getLogicalFuncResult(_intValuesSV[j], results[j]);
       }
     }
-    return _results;
+    return _intValuesSV;
   }
 
   abstract int getLogicalFuncResult(int left, int right);
