@@ -34,6 +34,7 @@ import org.apache.datasketches.theta.Sketch;
 import org.apache.datasketches.theta.Union;
 import org.apache.datasketches.theta.UpdateSketch;
 import org.apache.datasketches.theta.UpdateSketchBuilder;
+import org.apache.pinot.common.request.Expression;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FilterContext;
 import org.apache.pinot.common.request.context.FunctionContext;
@@ -100,7 +101,7 @@ public class DistinctCountThetaSketchAggregationFunction
       ExpressionContext paramsExpression = arguments.get(1);
       Preconditions.checkArgument(paramsExpression.getType() == ExpressionContext.Type.LITERAL,
           "Second argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal (parameters)");
-      Parameters parameters = new Parameters(paramsExpression.getLiteral());
+      Parameters parameters = new Parameters(paramsExpression.getLiteralString());
       int nominalEntries = parameters.getNominalEntries();
       _updateSketchBuilder.setNominalEntries(nominalEntries);
       _setOperationBuilder.setNominalEntries(nominalEntries);
@@ -130,7 +131,7 @@ public class DistinctCountThetaSketchAggregationFunction
             "Third to second last argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal "
                 + "(filter expression)");
         FilterContext filter =
-            RequestContextUtils.getFilter(CalciteSqlParser.compileToExpression(filterExpression.getLiteral()));
+            RequestContextUtils.getFilter(CalciteSqlParser.compileToExpression(filterExpression.getLiteralString()));
         // NOTE: Collect expressions before constructing the FilterInfo so that expressionIndexMap always include the
         //       expressions in the filter.
         collectExpressions(filter, _inputExpressions, expressionIndexMap);
@@ -142,8 +143,9 @@ public class DistinctCountThetaSketchAggregationFunction
       Preconditions.checkArgument(postAggregationExpression.getType() == ExpressionContext.Type.LITERAL,
           "Last argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal (post-aggregation "
               + "expression)");
-      _postAggregationExpression = RequestContextUtils.getExpression(
-          CalciteSqlParser.compileToExpression(postAggregationExpression.getLiteral()));
+      Expression expr = CalciteSqlParser.compileToExpression(postAggregationExpression.getLiteralString());
+      _postAggregationExpression = RequestContextUtils.getExpression(expr);
+
 
       // Validate the post-aggregation expression
       _includeDefaultSketch = validatePostAggregationExpression(_postAggregationExpression, _filterEvaluators.size());
@@ -1051,8 +1053,7 @@ public class DistinctCountThetaSketchAggregationFunction
    */
   private static boolean validatePostAggregationExpression(ExpressionContext expression, int numFilters) {
     Preconditions.checkArgument(expression.getType() != ExpressionContext.Type.LITERAL,
-        "Post-aggregation expression should not contain literal expression: %s", expression.getLiteral());
-
+        "Post-aggregation expression should not contain literal expression: %s", expression.toString());
     if (expression.getType() == ExpressionContext.Type.IDENTIFIER) {
       int sketchId = extractSketchId(expression.getIdentifier());
       Preconditions.checkArgument(sketchId <= numFilters, "Sketch id: %s exceeds number of filters: %s", sketchId,
