@@ -103,6 +103,7 @@ public final class ListenerConfigUtil {
 
     listeners.addAll(buildListenerConfigs(controllerConf, "controller", tlsDefaults));
 
+    Preconditions.checkState(!listeners.isEmpty(), "Missing listener configs");
     return listeners;
   }
 
@@ -155,14 +156,22 @@ public final class ListenerConfigUtil {
   public static List<ListenerConfig> buildMinionConfigs(PinotConfiguration minionConf) {
     List<ListenerConfig> listeners = new ArrayList<>();
 
-    int port = minionConf.getProperty(CommonConstants.Helix.KEY_OF_MINION_PORT,
-        CommonConstants.Minion.DEFAULT_HELIX_PORT);
-    listeners.add(new ListenerConfig(CommonConstants.HTTP_PROTOCOL, DEFAULT_HOST, port,
-        CommonConstants.HTTP_PROTOCOL, new TlsConfig()));
+    String portString = minionConf.getProperty(CommonConstants.Helix.KEY_OF_MINION_PORT);
+    if (portString != null) {
+      listeners.add(new ListenerConfig(CommonConstants.HTTP_PROTOCOL, DEFAULT_HOST, Integer.parseInt(portString),
+          CommonConstants.HTTP_PROTOCOL, new TlsConfig()));
+    }
 
     TlsConfig tlsDefaults = TlsUtils.extractTlsConfig(minionConf, CommonConstants.Minion.MINION_TLS_PREFIX);
     listeners.addAll(buildListenerConfigs(minionConf, "pinot.minion.adminapi", tlsDefaults));
 
+    // support legacy behavior < 0.7.0
+    if (listeners.isEmpty()) {
+      listeners.add(
+          new ListenerConfig(CommonConstants.HTTP_PROTOCOL, DEFAULT_HOST, CommonConstants.Minion.DEFAULT_HELIX_PORT,
+              CommonConstants.HTTP_PROTOCOL, new TlsConfig()));
+    }
+
     return listeners;
   }
 
