@@ -24,11 +24,16 @@ import java.util.List;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FunctionContext;
 import org.apache.pinot.common.request.context.RequestContextUtils;
+import org.apache.pinot.core.operator.transform.TransformResultMetadata;
+import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.exception.BadQueryRequestException;
-import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.assertFalse;
+import static org.testng.Assert.assertTrue;
+
 
 /**
  * LogicalOperatorTransformFunctionTest abstracts common test methods for:
@@ -38,9 +43,9 @@ import org.testng.annotations.Test;
  */
 public abstract class LogicalOperatorTransformFunctionTest extends BaseTransformFunctionTest {
 
-  abstract int getExpectedValue(boolean left, boolean right);
+  abstract boolean getExpectedValue(boolean left, boolean right);
 
-  abstract String getFuncName();
+  abstract String getFunctionName();
 
   @Test
   public void testLogicalOperatorTransformFunction() {
@@ -48,27 +53,31 @@ public abstract class LogicalOperatorTransformFunctionTest extends BaseTransform
         RequestContextUtils.getExpression(String.format("EQUALS(%s, %d)", INT_SV_COLUMN, _intSVValues[0]));
     ExpressionContext longEqualsExpr =
         RequestContextUtils.getExpression(String.format("EQUALS(%s, %d)", LONG_SV_COLUMN, _longSVValues[0]));
+    String functionName = getFunctionName();
     ExpressionContext expression = ExpressionContext.forFunction(
-        new FunctionContext(FunctionContext.Type.TRANSFORM, getFuncName(),
+        new FunctionContext(FunctionContext.Type.TRANSFORM, functionName,
             Arrays.asList(intEqualsExpr, longEqualsExpr)));
     TransformFunction transformFunction = TransformFunctionFactory.get(expression, _dataSourceMap);
-    Assert.assertEquals(transformFunction.getName(), getFuncName().toLowerCase());
-    int[] expectedIntValues = new int[NUM_ROWS];
+    assertEquals(transformFunction.getName(), functionName);
+    TransformResultMetadata resultMetadata = transformFunction.getResultMetadata();
+    assertEquals(resultMetadata.getDataType(), FieldSpec.DataType.BOOLEAN);
+    assertTrue(resultMetadata.isSingleValue());
+    assertFalse(resultMetadata.hasDictionary());
+    boolean[] expectedValues = new boolean[NUM_ROWS];
     for (int i = 0; i < NUM_ROWS; i++) {
-      expectedIntValues[i] = getExpectedValue(_intSVValues[i] == _intSVValues[0], _longSVValues[i] == _longSVValues[0]);
+      expectedValues[i] = getExpectedValue(_intSVValues[i] == _intSVValues[0], _longSVValues[i] == _longSVValues[0]);
     }
-    testTransformFunction(transformFunction, expectedIntValues);
+    testTransformFunction(transformFunction, expectedValues);
   }
 
   @Test(dataProvider = "testIllegalArguments", expectedExceptions = {BadQueryRequestException.class})
   public void testIllegalArguments(String[] expressions) {
     List<ExpressionContext> expressionContextList = new ArrayList<>();
-    for (int i = 0; i < expressions.length; i++) {
-      expressionContextList.add(RequestContextUtils.getExpression(expressions[i]));
+    for (String expression : expressions) {
+      expressionContextList.add(RequestContextUtils.getExpression(expression));
     }
-    TransformFunctionFactory.get(ExpressionContext
-            .forFunction(new FunctionContext(FunctionContext.Type.TRANSFORM, getFuncName(), expressionContextList)),
-        _dataSourceMap);
+    TransformFunctionFactory.get(ExpressionContext.forFunction(
+        new FunctionContext(FunctionContext.Type.TRANSFORM, getFunctionName(), expressionContextList)), _dataSourceMap);
   }
 
   @DataProvider(name = "testIllegalArguments")
