@@ -39,9 +39,12 @@ import org.apache.pinot.segment.local.segment.readers.GenericRowRecordReader;
 import org.apache.pinot.segment.local.segment.readers.PinotSegmentRecordReader;
 import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
 import org.apache.pinot.spi.config.table.ColumnPartitionConfig;
+import org.apache.pinot.spi.config.table.FieldConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
-import org.apache.pinot.spi.data.FieldSpec;
+import org.apache.pinot.spi.config.table.TimestampConfig;
+import org.apache.pinot.spi.config.table.TimestampIndexGranularity;
+import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.readers.GenericRow;
 import org.apache.pinot.spi.data.readers.RecordReader;
@@ -62,19 +65,33 @@ import static org.testng.Assert.assertTrue;
 public class SegmentMapperTest {
   private static final File TEMP_DIR = new File(FileUtils.getTempDirectory(), "SegmentMapperTest");
 
-  private final TableConfig _tableConfig =
-      new TableConfigBuilder(TableType.OFFLINE).setTableName("myTable").setTimeColumnName("timeValue")
-          .setNullHandlingEnabled(true).build();
-  private final Schema _schema = new Schema.SchemaBuilder().setSchemaName("myTable")
-      .addSingleValueDimension("campaign", FieldSpec.DataType.STRING, "xyz").addMetric("clicks", FieldSpec.DataType.INT)
-      .addDateTime("timeValue", FieldSpec.DataType.LONG, "1:MILLISECONDS:EPOCH", "1:MILLISECONDS").build();
-  private final List<Object[]> _rawData = Arrays
-      .asList(new Object[]{"abc", 1000, 1597719600000L}, new Object[]{"pqr", 2000, 1597773600000L},
+  private static final TableConfigBuilder TABLE_CONFIG_BUILDER =
+      new TableConfigBuilder(TableType.OFFLINE).setTableName("myTable").setTimeColumnName("ts")
+          .setNullHandlingEnabled(true).setFieldConfigList(Collections.singletonList(
+              new FieldConfig("ts", FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TIMESTAMP, null, null,
+                  new TimestampConfig(Collections.singletonList(TimestampIndexGranularity.DAY)), null)));
+  private static final Schema.SchemaBuilder SCHEMA_BUILDER =
+      new Schema.SchemaBuilder().setSchemaName("myTable").addSingleValueDimension("campaign", DataType.STRING, "xyz")
+          .addMetric("clicks", DataType.INT).addDateTime("ts", DataType.TIMESTAMP, "TIMESTAMP", "1:MILLISECONDS");
+  private static final List<Object[]> RAW_DATA =
+      Arrays.asList(new Object[]{"abc", 1000, 1597719600000L}, new Object[]{"pqr", 2000, 1597773600000L},
           new Object[]{"abc", 1000, 1597777200000L}, new Object[]{"abc", 4000, 1597795200000L},
           new Object[]{"abc", 3000, 1597802400000L}, new Object[]{"pqr", 1000, 1597838400000L},
           new Object[]{null, 4000, 1597856400000L}, new Object[]{"pqr", 1000, 1597878000000L},
           new Object[]{"abc", 7000, 1597881600000L}, new Object[]{null, 6000, 1597892400000L});
 
+  public TableConfig getTableConfig() {
+    return TABLE_CONFIG_BUILDER.build();
+  }
+
+  public Schema getSchema() {
+    return SCHEMA_BUILDER.build();
+  }
+
+  public List<Object[]> getRawData() {
+    return RAW_DATA;
+  }
+
   private File _indexDir;
 
   @BeforeClass
@@ -89,7 +106,7 @@ public class SegmentMapperTest {
 
     // Create test data
     List<GenericRow> inputRows = new ArrayList<>();
-    for (Object[] rawRow : _rawData) {
+    for (Object[] rawRow : getRawData()) {
       GenericRow row = new GenericRow();
       if (rawRow[0] != null) {
         row.putValue("campaign", rawRow[0]);
@@ -97,13 +114,13 @@ public class SegmentMapperTest {
         row.putDefaultNullValue("campaign", "xyz");
       }
       row.putValue("clicks", rawRow[1]);
-      row.putValue("timeValue", rawRow[2]);
+      row.putValue("ts", rawRow[2]);
       inputRows.add(row);
     }
 
     // Create test segment
     RecordReader recordReader = new GenericRowRecordReader(inputRows);
-    SegmentGeneratorConfig segmentGeneratorConfig = new SegmentGeneratorConfig(_tableConfig, _schema);
+    SegmentGeneratorConfig segmentGeneratorConfig = new SegmentGeneratorConfig(getTableConfig(), getSchema());
     segmentGeneratorConfig.setOutDir(inputSegmentDir.getAbsolutePath());
     SegmentIndexCreationDriverImpl driver = new SegmentIndexCreationDriverImpl();
     driver.init(segmentGeneratorConfig, recordReader);
@@ -154,7 +171,9 @@ public class SegmentMapperTest {
         Object[] expectedValues = expectedRecords.get(i);
         assertEquals(buffer.getValue("campaign"), expectedValues[0]);
         assertEquals(buffer.getValue("clicks"), expectedValues[1]);
-        assertEquals(buffer.getValue("timeValue"), expectedValues[2]);
+        assertEquals(buffer.getValue("ts"), expectedValues[2]);
+        // TIMESTAMP index
+        assertEquals(buffer.getValue("$ts$DAY"), (long) expectedValues[2] / 86400000 * 86400000);
         // Default null value
         if (expectedValues[0].equals("xyz")) {
           assertEquals(buffer.getNullValueFields(), Collections.singleton("campaign"));
@@ -172,8 +191,8 @@ public class SegmentMapperTest {
    */
   @DataProvider(name = "segmentMapperConfigProvider")
   public Object[][] segmentMapperConfigProvider() {
-    List<Object[]> outputData = Arrays
-        .asList(new Object[]{"abc", 1000, 1597719600000L}, new Object[]{"pqr", 2000, 1597773600000L},
+    List<Object[]> outputData =
+        Arrays.asList(new Object[]{"abc", 1000, 1597719600000L}, new Object[]{"pqr", 2000, 1597773600000L},
             new Object[]{"abc", 1000, 1597777200000L}, new Object[]{"abc", 4000, 1597795200000L},
             new Object[]{"abc", 3000, 1597802400000L}, new Object[]{"pqr", 1000, 1597838400000L},
             new Object[]{"xyz", 4000, 1597856400000L}, new Object[]{"pqr", 1000, 1597878000000L},
@@ -183,14 +202,14 @@ public class SegmentMapperTest {
 
     // Default configs
     SegmentProcessorConfig config0 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema()).build();
     Map<String, List<Object[]>> expectedRecords0 = Collections.singletonMap("0", outputData);
     inputs.add(new Object[]{config0, expectedRecords0});
 
     // Round-robin partitioner
     SegmentProcessorConfig config1 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setPartitionerConfigs(
-            Collections.singletonList(
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setPartitionerConfigs(Collections.singletonList(
                 new PartitionerConfig.Builder().setPartitionerType(PartitionerFactory.PartitionerType.ROUND_ROBIN)
                     .setNumPartitions(3).build())).build();
     Map<String, List<Object[]>> expectedRecords1 = new HashMap<>();
@@ -201,8 +220,8 @@ public class SegmentMapperTest {
 
     // Partition by campaign
     SegmentProcessorConfig config2 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setPartitionerConfigs(
-            Collections.singletonList(
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setPartitionerConfigs(Collections.singletonList(
                 new PartitionerConfig.Builder().setPartitionerType(PartitionerFactory.PartitionerType.COLUMN_VALUE)
                     .setColumnName("campaign").build())).build();
     Map<String, List<Object[]>> expectedRecords2 =
@@ -211,23 +230,22 @@ public class SegmentMapperTest {
 
     // Transform function partition
     SegmentProcessorConfig config3 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setPartitionerConfigs(
-            Collections.singletonList(new PartitionerConfig.Builder()
-                .setPartitionerType(PartitionerFactory.PartitionerType.TRANSFORM_FUNCTION)
-                .setTransformFunction("toEpochDays(timeValue)").build())).build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setPartitionerConfigs(Collections.singletonList(new PartitionerConfig.Builder().setPartitionerType(
+                PartitionerFactory.PartitionerType.TRANSFORM_FUNCTION).setTransformFunction("toEpochDays(ts)").build()))
+            .build();
     Map<String, List<Object[]>> expectedRecords3 =
         outputData.stream().collect(Collectors.groupingBy(r -> "0_" + ((long) r[2] / 86400000), Collectors.toList()));
     inputs.add(new Object[]{config3, expectedRecords3});
 
     // Partition by column and then table column partition config
     SegmentProcessorConfig config4 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setPartitionerConfigs(
-            Arrays.asList(
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setPartitionerConfigs(Arrays.asList(
                 new PartitionerConfig.Builder().setPartitionerType(PartitionerFactory.PartitionerType.COLUMN_VALUE)
-                    .setColumnName("campaign").build(), new PartitionerConfig.Builder()
-                    .setPartitionerType(PartitionerFactory.PartitionerType.TABLE_PARTITION_CONFIG)
-                    .setColumnName("clicks").setColumnPartitionConfig(new ColumnPartitionConfig("Modulo", 3, null))
-                    .build())).build();
+                    .setColumnName("campaign").build(), new PartitionerConfig.Builder().setPartitionerType(
+                        PartitionerFactory.PartitionerType.TABLE_PARTITION_CONFIG).setColumnName("clicks")
+                    .setColumnPartitionConfig(new ColumnPartitionConfig("Modulo", 3, null)).build())).build();
     Map<String, List<Object[]>> expectedRecords4 = new HashMap<>();
     for (Object[] record : outputData) {
       String partition = "0_" + record[0] + "_" + ((int) record[1] % 3);
@@ -238,9 +256,10 @@ public class SegmentMapperTest {
 
     // Time handling - filter out certain times
     SegmentProcessorConfig config5 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setTimeHandlerConfig(
-            new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setTimeRange(1597795200000L, 1597881600000L).build())
-            .build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setTimeHandlerConfig(
+                new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setTimeRange(1597795200000L, 1597881600000L)
+                    .build()).build();
     Map<String, List<Object[]>> expectedRecords5 =
         outputData.stream().filter(r -> ((long) r[2]) >= 1597795200000L && ((long) r[2]) < 1597881600000L)
             .collect(Collectors.groupingBy(r -> "0", Collectors.toList()));
@@ -248,8 +267,9 @@ public class SegmentMapperTest {
 
     // Time handling - round time to nearest day
     SegmentProcessorConfig config6 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setTimeHandlerConfig(
-            new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setRoundBucketMs(86400000).build()).build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setTimeHandlerConfig(
+                new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setRoundBucketMs(86400000).build()).build();
     Map<String, List<Object[]>> expectedRecords6 =
         outputData.stream().map(r -> new Object[]{r[0], r[1], (((long) r[2]) / 86400000) * 86400000})
             .collect(Collectors.groupingBy(r -> "0", Collectors.toList()));
@@ -257,17 +277,19 @@ public class SegmentMapperTest {
 
     // Time handling - partition time by day
     SegmentProcessorConfig config7 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setTimeHandlerConfig(
-            new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setPartitionBucketMs(86400000).build()).build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setTimeHandlerConfig(
+                new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setPartitionBucketMs(86400000).build()).build();
     Map<String, List<Object[]>> expectedRecords7 = outputData.stream()
         .collect(Collectors.groupingBy(r -> Long.toString(((long) r[2]) / 86400000), Collectors.toList()));
     inputs.add(new Object[]{config7, expectedRecords7});
 
     // Time handling - filter out certain times, round time to nearest hour, partition by day
     SegmentProcessorConfig config8 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setTimeHandlerConfig(
-            new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setTimeRange(1597795200000L, 1597881600000L)
-                .setRoundBucketMs(3600000).setPartitionBucketMs(86400000).build()).build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setTimeHandlerConfig(
+                new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setTimeRange(1597795200000L, 1597881600000L)
+                    .setRoundBucketMs(3600000).setPartitionBucketMs(86400000).build()).build();
     Map<String, List<Object[]>> expectedRecords8 =
         outputData.stream().filter(r -> ((long) r[2]) >= 1597795200000L && ((long) r[2]) < 1597881600000L)
             .map(r -> new Object[]{r[0], r[1], (((long) r[2]) / 3600000) * 3600000})
@@ -276,9 +298,10 @@ public class SegmentMapperTest {
 
     // Time handling - negate filter with certain times
     SegmentProcessorConfig config9 =
-        new SegmentProcessorConfig.Builder().setTableConfig(_tableConfig).setSchema(_schema).setTimeHandlerConfig(
-            new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setTimeRange(1597795200000L, 1597881600000L)
-                .setNegateWindowFilter(true).build()).build();
+        new SegmentProcessorConfig.Builder().setTableConfig(getTableConfig()).setSchema(getSchema())
+            .setTimeHandlerConfig(
+                new TimeHandlerConfig.Builder(TimeHandler.Type.EPOCH).setTimeRange(1597795200000L, 1597881600000L)
+                    .setNegateWindowFilter(true).build()).build();
     Map<String, List<Object[]>> expectedRecords9 =
         outputData.stream().filter(r -> !(((long) r[2]) >= 1597795200000L && ((long) r[2]) < 1597881600000L))
             .collect(Collectors.groupingBy(r -> "0", Collectors.toList()));
