@@ -70,6 +70,7 @@ import org.testng.annotations.Test;
 
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNotNull;
+import static org.testng.Assert.assertNull;
 
 
 /**
@@ -93,6 +94,8 @@ abstract class BaseStarTreeV2Test<R, A> {
   private static final String DIMENSION_D2 = "d2";
   private static final int DIMENSION_CARDINALITY = 100;
   private static final String METRIC = "m";
+
+  // Supported filters
   private static final String QUERY_FILTER_AND = " WHERE d1 = 0 AND d2 < 10";
   // StarTree supports OR predicates only on a single dimension
   private static final String QUERY_FILTER_OR = " WHERE d1 > 10 OR d1 < 50";
@@ -102,10 +105,18 @@ abstract class BaseStarTreeV2Test<R, A> {
   private static final String QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS_THREE_PREDICATES =
       " WHERE (d2 > 95 OR d2 < 25) AND (d1 > 10 OR d1 < 50)";
   private static final String QUERY_FILTER_COMPLEX_OR_SINGLE_DIMENSION = " WHERE d1 = 95 AND (d1 > 90 OR d1 < 100)";
+
+  // Unsupported filters
+  private static final String QUERY_FILTER_OR_MULTIPLE_DIMENSIONS = " WHERE d1 > 10 OR d2 < 50";
+  private static final String QUERY_FILTER_OR_ON_AND = " WHERE (d1 > 10 AND d1 < 50) OR d1 < 50";
+  private static final String QUERY_FILTER_OR_ON_NOT = " WHERE (NOT d1 > 10) OR d1 < 50";
+
   private static final String QUERY_GROUP_BY = " GROUP BY d2";
+  private static final String FILTER_AGG_CLAUSE = " FILTER(WHERE d1 > 10)";
 
   private ValueAggregator _valueAggregator;
   private DataType _aggregatedValueType;
+  private String _aggregation;
   private IndexSegment _indexSegment;
   private StarTreeV2 _starTreeV2;
 
@@ -114,6 +125,16 @@ abstract class BaseStarTreeV2Test<R, A> {
       throws Exception {
     _valueAggregator = getValueAggregator();
     _aggregatedValueType = _valueAggregator.getAggregatedValueType();
+    AggregationFunctionType aggregationType = _valueAggregator.getAggregationType();
+    if (aggregationType == AggregationFunctionType.COUNT) {
+      _aggregation = "COUNT(*)";
+    } else if (aggregationType == AggregationFunctionType.PERCENTILEEST
+        || aggregationType == AggregationFunctionType.PERCENTILETDIGEST) {
+      // Append a percentile number for percentile functions
+      _aggregation = String.format("%s(%s, 50)", aggregationType.getName(), METRIC);
+    } else {
+      _aggregation = String.format("%s(%s)", aggregationType.getName(), METRIC);
+    }
 
     Schema.SchemaBuilder schemaBuilder = new Schema.SchemaBuilder().addSingleValueDimension(DIMENSION_D1, DataType.INT)
         .addSingleValueDimension(DIMENSION_D2, DataType.INT);
@@ -160,36 +181,30 @@ abstract class BaseStarTreeV2Test<R, A> {
     _starTreeV2 = _indexSegment.getStarTrees().get(0);
   }
 
+  @Test
+  public void testUnsupportedFilters() {
+    String query = String.format("SELECT %s FROM %s", _aggregation, TABLE_NAME);
+    testUnsupportedFilter(query + QUERY_FILTER_OR_MULTIPLE_DIMENSIONS);
+    testUnsupportedFilter(query + QUERY_FILTER_OR_ON_AND);
+    testUnsupportedFilter(query + QUERY_FILTER_OR_ON_NOT);
+  }
+
   @Test
   public void testQueries()
       throws IOException {
-    AggregationFunctionType aggregationType = _valueAggregator.getAggregationType();
-    String aggregation;
-    if (aggregationType == AggregationFunctionType.COUNT) {
-      aggregation = "COUNT(*)";
-    } else if (aggregationType == AggregationFunctionType.PERCENTILEEST
-        || aggregationType == AggregationFunctionType.PERCENTILETDIGEST) {
-      // Append a percentile number for percentile functions
-      aggregation = String.format("%s50(%s)", aggregationType.getName(), METRIC);
-    } else {
-      aggregation = String.format("%s(%s)", aggregationType.getName(), METRIC);
+    String nonFilteredQuery = String.format("SELECT %s FROM %s", _aggregation, TABLE_NAME);
+    String filteredQuery = String.format("SELECT %s%s FROM %s", _aggregation, FILTER_AGG_CLAUSE, TABLE_NAME);
+    for (String query : Arrays.asList(nonFilteredQuery, filteredQuery)) {
+      testQuery(query);
+      testQuery(query + QUERY_FILTER_AND);
+      testQuery(query + QUERY_FILTER_OR);
+      testQuery(query + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS);
+      testQuery(query + QUERY_FILTER_COMPLEX_AND_MULTIPLE_DIMENSIONS_THREE_PREDICATES);
+      testQuery(query + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS_THREE_PREDICATES);
+      testQuery(query + QUERY_FILTER_COMPLEX_OR_SINGLE_DIMENSION);
     }
-
-    String baseQuery = String.format("SELECT %s FROM %s", aggregation, TABLE_NAME);
-    testQuery(baseQuery);
-    testQuery(baseQuery + QUERY_FILTER_AND);
-    testQuery(baseQuery + QUERY_FILTER_OR);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_AND_MULTIPLE_DIMENSIONS_THREE_PREDICATES);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS_THREE_PREDICATES);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_OR_SINGLE_DIMENSION);
-    testQuery(baseQuery + QUERY_GROUP_BY);
-    testQuery(baseQuery + QUERY_FILTER_AND + QUERY_GROUP_BY);
-    testQuery(baseQuery + QUERY_FILTER_OR + QUERY_GROUP_BY);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS + QUERY_GROUP_BY);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS_THREE_PREDICATES + QUERY_GROUP_BY);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_AND_MULTIPLE_DIMENSIONS_THREE_PREDICATES + QUERY_GROUP_BY);
-    testQuery(baseQuery + QUERY_FILTER_COMPLEX_OR_SINGLE_DIMENSION + QUERY_GROUP_BY);
+    // TODO: Test group-by on filtered query after supporting it
+    testQuery(nonFilteredQuery + QUERY_GROUP_BY);
   }
 
   @AfterClass
@@ -199,9 +214,19 @@ abstract class BaseStarTreeV2Test<R, A> {
     FileUtils.deleteDirectory(TEMP_DIR);
   }
 
-  void testQuery(String query)
+  private void testUnsupportedFilter(String query) {
+    QueryContext queryContext = QueryContextConverterUtils.getQueryContext(query);
+    FilterPlanNode filterPlanNode = new FilterPlanNode(_indexSegment, queryContext);
+    filterPlanNode.run();
+    Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap =
+        StarTreeUtils.extractPredicateEvaluatorsMap(_indexSegment, queryContext.getFilter(),
+            filterPlanNode.getPredicateEvaluators());
+    assertNull(predicateEvaluatorsMap);
+  }
+
+  private void testQuery(String query)
       throws IOException {
-    QueryContext queryContext = QueryContextConverterUtils.getQueryContextFromSQL(query);
+    QueryContext queryContext = QueryContextConverterUtils.getQueryContext(query);
 
     // Aggregations
     AggregationFunction[] aggregationFunctions = queryContext.getAggregationFunctions();
@@ -227,12 +252,12 @@ abstract class BaseStarTreeV2Test<R, A> {
     filterPlanNode.run();
     Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap =
         StarTreeUtils.extractPredicateEvaluatorsMap(_indexSegment, queryContext.getFilter(),
-            filterPlanNode.getPredicateEvaluatorMap());
+            filterPlanNode.getPredicateEvaluators());
     assertNotNull(predicateEvaluatorsMap);
 
     // Extract values with star-tree
     PlanNode starTreeFilterPlanNode =
-        new StarTreeFilterPlanNode(_starTreeV2, predicateEvaluatorsMap, groupByColumnSet, null);
+        new StarTreeFilterPlanNode(queryContext, _starTreeV2, predicateEvaluatorsMap, groupByColumnSet);
     List<ForwardIndexReader> starTreeAggregationColumnReaders = new ArrayList<>(numAggregations);
     for (AggregationFunctionColumnPair aggregationFunctionColumnPair : aggregationFunctionColumnPairs) {
       starTreeAggregationColumnReaders.add(
