@@ -27,10 +27,11 @@ import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
 import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.core.operator.ExecutionStatistics;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
 import org.apache.pinot.core.operator.query.EmptySelectionOperator;
-import org.apache.pinot.core.operator.query.SelectionOrderByOperator;
-import org.apache.pinot.core.query.request.context.ThreadTimer;
+import org.apache.pinot.core.operator.query.SelectionPartiallyOrderedByAscOperator;
+import org.apache.pinot.core.operator.query.SelectionPartiallyOrderedByDescOperation;
+import org.apache.pinot.spi.accounting.ThreadResourceUsageProvider;
 import org.testng.annotations.Test;
 
 import static org.testng.Assert.assertEquals;
@@ -49,7 +50,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
 
     // Test query without filter
     EmptySelectionOperator emptySelectionOperator = getOperator(SELECT_STAR_QUERY + limit);
-    IntermediateResultsBlock resultsBlock = emptySelectionOperator.nextBlock();
+    SelectionResultsBlock resultsBlock = emptySelectionOperator.nextBlock();
     ExecutionStatistics executionStatistics = emptySelectionOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 0L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -62,7 +63,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column11"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column11")), ColumnDataType.STRING);
-    assertTrue(resultsBlock.getSelectionResult().isEmpty());
+    assertTrue(resultsBlock.getRows().isEmpty());
 
     // Test query with filter
     emptySelectionOperator = getOperator(SELECT_STAR_QUERY + FILTER + limit);
@@ -80,7 +81,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column11"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column11")), ColumnDataType.STRING);
-    assertTrue(resultsBlock.getSelectionResult().isEmpty());
+    assertTrue(resultsBlock.getRows().isEmpty());
   }
 
   @Test
@@ -88,8 +89,8 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     String query = "SELECT daysSinceEpoch FROM testTable WHERE "
         + "dateTimeConvert(daysSinceEpoch, '1:DAYS:EPOCH', '1:MILLISECONDS:EPOCH', '1:MILLISECONDS') > ago('P1D') "
         + "ORDER BY daysSinceEpoch LIMIT 10";
-    SelectionOrderByOperator selectionOrderByOperator = getOperator(query);
-    IntermediateResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOrderByOperator = getOperator(query);
+    SelectionResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
     verifySelectionOrderByAgoFunctionResult(resultsBlock);
 
     query = "SELECT daysSinceEpoch from testTable WHERE fromEpochDays(daysSinceEpoch) > ago('P1D') "
@@ -99,7 +100,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     verifySelectionOrderByAgoFunctionResult(resultsBlock);
   }
 
-  private void verifySelectionOrderByAgoFunctionResult(IntermediateResultsBlock resultsBlock) {
+  private void verifySelectionOrderByAgoFunctionResult(SelectionResultsBlock resultsBlock) {
     DataSchema selectionDataSchema = resultsBlock.getDataSchema();
     Map<String, Integer> columnIndexMap = computeColumnNameToIndexMap(selectionDataSchema);
     assertEquals(selectionDataSchema.size(), 1);
@@ -107,7 +108,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("daysSinceEpoch"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("daysSinceEpoch")), ColumnDataType.INT);
 
-    PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    PriorityQueue<Object[]> selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     for (Object[] row : selectionResult) {
       assertEquals(row.length, 1);
@@ -118,8 +119,8 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
   @Test
   public void testSelectStar() {
     // Test query without filter
-    BaseOperator<IntermediateResultsBlock> selectionOnlyOperator = getOperator(SELECT_STAR_QUERY);
-    IntermediateResultsBlock resultsBlock = selectionOnlyOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOnlyOperator = getOperator(SELECT_STAR_QUERY);
+    SelectionResultsBlock resultsBlock = selectionOnlyOperator.nextBlock();
     ExecutionStatistics executionStatistics = selectionOnlyOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 10L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -133,7 +134,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column11"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column11")), ColumnDataType.STRING);
-    List<Object[]> selectionResult = (List<Object[]>) resultsBlock.getSelectionResult();
+    List<Object[]> selectionResult = (List<Object[]>) resultsBlock.getRows();
     assertEquals(selectionResult.size(), 10);
     Object[] firstRow = selectionResult.get(0);
     assertEquals(firstRow.length, 11);
@@ -145,7 +146,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     resultsBlock = selectionOnlyOperator.nextBlock();
     executionStatistics = selectionOnlyOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 10L);
-    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 48241L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 48204L);
     assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 110L);
     assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
     selectionDataSchema = resultsBlock.getDataSchema();
@@ -156,7 +157,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column11"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column11")), ColumnDataType.STRING);
-    selectionResult = (List<Object[]>) resultsBlock.getSelectionResult();
+    selectionResult = (List<Object[]>) resultsBlock.getRows();
     assertEquals(selectionResult.size(), 10);
     firstRow = selectionResult.get(0);
     assertEquals(firstRow.length, 11);
@@ -167,8 +168,8 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
   @Test
   public void testSelectionOnly() {
     // Test query without filter
-    BaseOperator<IntermediateResultsBlock> selectionOnlyOperator = getOperator(SELECTION_QUERY);
-    IntermediateResultsBlock resultsBlock = selectionOnlyOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOnlyOperator = getOperator(SELECTION_QUERY);
+    SelectionResultsBlock resultsBlock = selectionOnlyOperator.nextBlock();
     ExecutionStatistics executionStatistics = selectionOnlyOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 10L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -182,7 +183,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column11"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column11")), ColumnDataType.STRING);
-    List<Object[]> selectionResult = (List<Object[]>) resultsBlock.getSelectionResult();
+    List<Object[]> selectionResult = (List<Object[]>) resultsBlock.getRows();
     assertEquals(selectionResult.size(), 10);
     Object[] firstRow = selectionResult.get(0);
     assertEquals(firstRow.length, 3);
@@ -194,7 +195,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     resultsBlock = selectionOnlyOperator.nextBlock();
     executionStatistics = selectionOnlyOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 10L);
-    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 48241L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 48204L);
     assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 30L);
     assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
     selectionDataSchema = resultsBlock.getDataSchema();
@@ -204,7 +205,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column11"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column11")), ColumnDataType.STRING);
-    selectionResult = (List<Object[]>) resultsBlock.getSelectionResult();
+    selectionResult = (List<Object[]>) resultsBlock.getRows();
     assertEquals(selectionResult.size(), 10);
     firstRow = selectionResult.get(0);
     assertEquals(firstRow.length, 3);
@@ -215,8 +216,8 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
   @Test
   public void testSelectionOrderBy() {
     // Test query without filter
-    BaseOperator<IntermediateResultsBlock> selectionOrderByOperator = getOperator(SELECTION_QUERY + ORDER_BY);
-    IntermediateResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOrderByOperator = getOperator(SELECTION_QUERY + ORDER_BY);
+    SelectionResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
     ExecutionStatistics executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -231,7 +232,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column1"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column6")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
-    PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    PriorityQueue<Object[]> selectionResult = resultsBlock.getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     Object[] lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 4);
@@ -243,7 +244,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     resultsBlock = selectionOrderByOperator.nextBlock();
     executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 6129L);
-    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 84134L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 63064L);
     // 6129 * (2 order-by columns) + 10 * (2 non-order-by columns)
     assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 12278L);
     assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
@@ -255,7 +256,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column1"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column6")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
-    selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    selectionResult = resultsBlock.getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 4);
@@ -263,11 +264,167 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertEquals(((Integer) lastRow[columnIndexMap.get("column1")]).intValue(), 462769197);
   }
 
+  @Test
+  public void testSelectionOrderBySortedColumn() {
+    // Test query order by single sorted column in ascending order
+    String orderBy = " ORDER BY column5";
+    BaseOperator<SelectionResultsBlock> selectionOrderByOperator = getOperator(SELECTION_QUERY + orderBy);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByAscOperator);
+    SelectionResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
+    ExecutionStatistics executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 10L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 10 * (3 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 30L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    DataSchema dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "column1", "column11"});
+    assertEquals(dataSchema.getColumnDataTypes(),
+        new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT, ColumnDataType.STRING});
+    PriorityQueue<Object[]> selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    Object[] lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 3);
+    assertEquals(lastRow[0], "gFuH");
+
+    // Test query order by single sorted column in descending order
+    orderBy = " ORDER BY column5 DESC";
+    selectionOrderByOperator = getOperator(SELECTION_QUERY + orderBy);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByDescOperation);
+    resultsBlock = selectionOrderByOperator.nextBlock();
+    executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 30000 * (3 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 90000L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "column1", "column11"});
+    assertEquals(dataSchema.getColumnDataTypes(),
+        new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT, ColumnDataType.STRING});
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 3);
+    assertEquals(lastRow[0], "gFuH");
+
+    // Test query order by all sorted columns in ascending order
+    String query = "SELECT column5, daysSinceEpoch FROM testTable ORDER BY column5, daysSinceEpoch";
+    selectionOrderByOperator = getOperator(query);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByAscOperator);
+    resultsBlock = selectionOrderByOperator.nextBlock();
+    executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 10L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 10 * (2 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 20L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "daysSinceEpoch"});
+    assertEquals(dataSchema.getColumnDataTypes(), new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT});
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 2);
+    assertEquals(lastRow[0], "gFuH");
+    assertEquals(lastRow[1], 126164076);
+
+    // Test query order by all sorted columns in descending order
+    query = "SELECT column5 FROM testTable ORDER BY column5 DESC, daysSinceEpoch DESC";
+    selectionOrderByOperator = getOperator(query);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByDescOperation);
+    resultsBlock = selectionOrderByOperator.nextBlock();
+    executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 30000 * (2 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 60000L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "daysSinceEpoch"});
+    assertEquals(dataSchema.getColumnDataTypes(), new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT});
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 2);
+    assertEquals(lastRow[0], "gFuH");
+    assertEquals(lastRow[1], 167572854);
+
+    // Test query order by one sorted column in ascending order, the other sorted column in descending order
+    query = "SELECT daysSinceEpoch FROM testTable ORDER BY column5, daysSinceEpoch DESC";
+    selectionOrderByOperator = getOperator(query);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByAscOperator);
+    resultsBlock = selectionOrderByOperator.nextBlock();
+    executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 30000 * (2 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 60000L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "daysSinceEpoch"});
+    assertEquals(dataSchema.getColumnDataTypes(), new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT});
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 2);
+    assertEquals(lastRow[0], "gFuH");
+    assertEquals(lastRow[1], 167572854);
+
+    // Test query order by one sorted column in ascending order, and some unsorted columns
+    query = "SELECT column1 FROM testTable ORDER BY column5, column6, column1";
+    selectionOrderByOperator = getOperator(query);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByAscOperator);
+    resultsBlock = selectionOrderByOperator.nextBlock();
+    executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 30000 * (3 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 90000L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "column6", "column1"});
+    assertEquals(dataSchema.getColumnDataTypes(),
+        new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT, ColumnDataType.INT});
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 3);
+    assertEquals(lastRow[0], "gFuH");
+    // Unsorted column values should be the same as ordering by their own
+    assertEquals(lastRow[1], 6043515);
+    assertEquals(lastRow[2], 10542595);
+
+    // Test query order by one sorted column in descending order, and some unsorted columns
+    query = "SELECT column6 FROM testTable ORDER BY column5 DESC, column6, column1";
+    selectionOrderByOperator = getOperator(query);
+    assertTrue(selectionOrderByOperator instanceof SelectionPartiallyOrderedByDescOperation);
+    resultsBlock = selectionOrderByOperator.nextBlock();
+    executionStatistics = selectionOrderByOperator.getExecutionStatistics();
+    assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
+    // 30000 * (3 columns)
+    assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 90000L);
+    assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
+    dataSchema = resultsBlock.getDataSchema();
+    assertEquals(dataSchema.getColumnNames(), new String[]{"column5", "column6", "column1"});
+    assertEquals(dataSchema.getColumnDataTypes(),
+        new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.INT, ColumnDataType.INT});
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
+    assertEquals(selectionResult.size(), 10);
+    lastRow = selectionResult.peek();
+    assertEquals(lastRow.length, 3);
+    assertEquals(lastRow[0], "gFuH");
+    // Unsorted column values should be the same as ordering by their own
+    assertEquals(lastRow[1], 6043515);
+    assertEquals(lastRow[2], 10542595);
+  }
+
   @Test
   public void testSelectStarOrderBy() {
     // Test query without filter
-    BaseOperator<IntermediateResultsBlock> selectionOrderByOperator = getOperator(SELECT_STAR_QUERY + ORDER_BY);
-    IntermediateResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOrderByOperator = getOperator(SELECT_STAR_QUERY + ORDER_BY);
+    SelectionResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
     ExecutionStatistics executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -283,7 +440,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column1"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column6")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
-    PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    PriorityQueue<Object[]> selectionResult = resultsBlock.getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     Object[] lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 11);
@@ -295,7 +452,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     resultsBlock = selectionOrderByOperator.nextBlock();
     executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 6129L);
-    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 84134L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 63064L);
     // 6129 * (2 order-by columns) + 10 * (9 non-order-by columns)
     assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 12348L);
     assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
@@ -308,7 +465,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column1"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column6")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
-    selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    selectionResult = resultsBlock.getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 11);
@@ -321,8 +478,8 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     String orderBy = " ORDER BY column5";
 
     // Test query without filter
-    BaseOperator<IntermediateResultsBlock> selectionOrderByOperator = getOperator(SELECT_STAR_QUERY + orderBy);
-    IntermediateResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOrderByOperator = getOperator(SELECT_STAR_QUERY + orderBy);
+    SelectionResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
     ExecutionStatistics executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 10L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -336,7 +493,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertEquals(selectionDataSchema.size(), 11);
     assertTrue(columnIndexMap.containsKey("column5"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column5")), ColumnDataType.STRING);
-    PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    PriorityQueue<Object[]> selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     Object[] lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 11);
@@ -347,7 +504,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     resultsBlock = selectionOrderByOperator.nextBlock();
     executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 10);
-    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 48241L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 48204L);
     // 10 * (11 columns)
     assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 110);
     assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
@@ -358,7 +515,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertEquals(selectionDataSchema.size(), 11);
     assertTrue(columnIndexMap.containsKey("column5"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column5")), ColumnDataType.STRING);
-    selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    selectionResult = resultsBlock.convertToPriorityQueueBased().getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 10);
     lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 11);
@@ -370,8 +527,8 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     String limit = " LIMIT 5000, 7000";
 
     // Test query without filter
-    BaseOperator<IntermediateResultsBlock> selectionOrderByOperator = getOperator(SELECT_STAR_QUERY + ORDER_BY + limit);
-    IntermediateResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
+    BaseOperator<SelectionResultsBlock> selectionOrderByOperator = getOperator(SELECT_STAR_QUERY + ORDER_BY + limit);
+    SelectionResultsBlock resultsBlock = selectionOrderByOperator.nextBlock();
     ExecutionStatistics executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 30000L);
     assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 0L);
@@ -387,7 +544,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column1"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column6")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
-    PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    PriorityQueue<Object[]> selectionResult = resultsBlock.getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 12000);
     Object[] lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 11);
@@ -399,7 +556,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     resultsBlock = selectionOrderByOperator.nextBlock();
     executionStatistics = selectionOrderByOperator.getExecutionStatistics();
     assertEquals(executionStatistics.getNumDocsScanned(), 6129L);
-    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 84134L);
+    assertEquals(executionStatistics.getNumEntriesScannedInFilter(), 63064L);
     // 6129 * (2 order-by columns) + 6129 * (9 non-order-by columns)
     assertEquals(executionStatistics.getNumEntriesScannedPostFilter(), 67419L);
     assertEquals(executionStatistics.getNumTotalDocs(), 30000L);
@@ -412,7 +569,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertTrue(columnIndexMap.containsKey("column1"));
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column6")), ColumnDataType.INT);
     assertEquals(selectionDataSchema.getColumnDataType(columnIndexMap.get("column1")), ColumnDataType.INT);
-    selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
+    selectionResult = resultsBlock.getRowsAsPriorityQueue();
     assertEquals(selectionResult.size(), 6129);
     lastRow = selectionResult.peek();
     assertEquals(lastRow.length, 11);
@@ -420,7 +577,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     assertEquals((int) lastRow[columnIndexMap.get("column1")], 335520083);
   }
 
-  private int getVirtualColumns(DataSchema selectionDataSchema) {
+  int getVirtualColumns(DataSchema selectionDataSchema) {
     int virtualCols = 0;
     for (int i = 0; i < selectionDataSchema.size(); i++) {
       if (selectionDataSchema.getColumnName(i).startsWith("$")) {
@@ -430,7 +587,7 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
     return virtualCols;
   }
 
-  private Map<String, Integer> computeColumnNameToIndexMap(DataSchema dataSchema) {
+  Map<String, Integer> computeColumnNameToIndexMap(DataSchema dataSchema) {
     Map<String, Integer> columnIndexMap = new HashMap<>();
 
     for (int i = 0; i < dataSchema.size(); i++) {
@@ -443,16 +600,16 @@ public class InnerSegmentSelectionSingleValueQueriesTest extends BaseSingleValue
   public void testThreadCpuTime() {
     String query = "SELECT * FROM testTable";
 
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(true);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(true);
     // NOTE: Need to check whether thread CPU time measurement is enabled because some environments might not support
     //       ThreadMXBean.getCurrentThreadCpuTime()
-    if (ThreadTimer.isThreadCpuTimeMeasurementEnabled()) {
+    if (ThreadResourceUsageProvider.isThreadCpuTimeMeasurementEnabled()) {
       BrokerResponseNative brokerResponse = getBrokerResponse(query);
       assertTrue(brokerResponse.getOfflineThreadCpuTimeNs() > 0);
       assertTrue(brokerResponse.getRealtimeThreadCpuTimeNs() > 0);
     }
 
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
     BrokerResponseNative brokerResponse = getBrokerResponse(query);
     assertEquals(brokerResponse.getOfflineThreadCpuTimeNs(), 0);
     assertEquals(brokerResponse.getRealtimeThreadCpuTimeNs(), 0);
