@@ -27,6 +27,7 @@ import org.apache.commons.io.FileUtils;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.segment.local.indexsegment.mutable.MutableSegmentImpl;
 import org.apache.pinot.segment.local.io.writer.impl.DirectMemoryManager;
+import org.apache.pinot.segment.local.realtime.converter.ColumnIndicesForRealtimeTable;
 import org.apache.pinot.segment.local.realtime.converter.RealtimeSegmentConverter;
 import org.apache.pinot.segment.local.realtime.impl.RealtimeSegmentConfig;
 import org.apache.pinot.segment.local.realtime.impl.RealtimeSegmentStatsHistory;
@@ -40,11 +41,12 @@ import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.TimeGranularitySpec;
-import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
-import org.testng.Assert;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.assertTrue;
+
 
 public class RealtimeSegmentConverterTest {
 
@@ -73,15 +75,14 @@ public class RealtimeSegmentConverterTest {
             new TimeGranularitySpec(FieldSpec.DataType.LONG, TimeUnit.DAYS, "col2")).build();
     String segmentName = "segment1";
     VirtualColumnProviderFactory.addBuiltInVirtualColumnsToSegmentSchema(schema, segmentName);
-    Assert.assertEquals(schema.getColumnNames().size(), 5);
+    assertEquals(schema.getColumnNames().size(), 5);
     Schema newSchema = RealtimeSegmentConverter.getUpdatedSchema(schema);
-    Assert.assertEquals(newSchema.getColumnNames().size(), 2);
+    assertEquals(newSchema.getColumnNames().size(), 2);
   }
 
   @Test
   public void testNoRecordsIndexed()
       throws Exception {
-
     File tmpDir = new File(TMP_DIR, "tmp_" + System.currentTimeMillis());
     TableConfig tableConfig =
         new TableConfigBuilder(TableType.OFFLINE).setTableName("testTable").setTimeColumnName(DATE_TIME_COLUMN)
@@ -122,23 +123,23 @@ public class RealtimeSegmentConverterTest {
     SegmentZKPropsConfig segmentZKPropsConfig = new SegmentZKPropsConfig();
     segmentZKPropsConfig.setStartOffset("1");
     segmentZKPropsConfig.setEndOffset("100");
+    ColumnIndicesForRealtimeTable cdc = new ColumnIndicesForRealtimeTable(indexingConfig.getSortedColumn().get(0),
+        indexingConfig.getInvertedIndexColumns(), null, null,
+        indexingConfig.getNoDictionaryColumns(), indexingConfig.getVarLengthDictionaryColumns());
     RealtimeSegmentConverter converter =
         new RealtimeSegmentConverter(mutableSegmentImpl, segmentZKPropsConfig, outputDir.getAbsolutePath(), schema,
-            tableNameWithType, tableConfig, segmentName, indexingConfig.getSortedColumn().get(0),
-            indexingConfig.getInvertedIndexColumns(), null, null, indexingConfig.getNoDictionaryColumns(),
-            indexingConfig.getVarLengthDictionaryColumns(), false);
-
+            tableNameWithType, tableConfig, segmentName, cdc, false);
     converter.build(SegmentVersion.v3, null);
-    SegmentMetadataImpl metadata = new SegmentMetadataImpl(new File(outputDir, segmentName));
-    Assert.assertEquals(metadata.getTotalDocs(), 0);
-    Assert.assertEquals(metadata.getTimeColumn(), DATE_TIME_COLUMN);
-    Assert.assertEquals(metadata.getTimeUnit(), TimeUnit.MILLISECONDS);
-    Assert.assertEquals(metadata.getStartTime(), metadata.getEndTime());
-    Assert.assertTrue(metadata.getAllColumns().containsAll(schema.getColumnNames()));
-    Assert.assertEquals(
-        metadata.getPropertiesConfiguration().getProperty(CommonConstants.Segment.Realtime.START_OFFSET), "1");
-    Assert.assertEquals(metadata.getPropertiesConfiguration().getProperty(CommonConstants.Segment.Realtime.END_OFFSET),
-        "100");
+
+    File indexDir = new File(outputDir, segmentName);
+    SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(indexDir);
+    assertEquals(segmentMetadata.getTotalDocs(), 0);
+    assertEquals(segmentMetadata.getTimeColumn(), DATE_TIME_COLUMN);
+    assertEquals(segmentMetadata.getTimeUnit(), TimeUnit.MILLISECONDS);
+    assertEquals(segmentMetadata.getStartTime(), segmentMetadata.getEndTime());
+    assertTrue(segmentMetadata.getAllColumns().containsAll(schema.getColumnNames()));
+    assertEquals(segmentMetadata.getStartOffset(), "1");
+    assertEquals(segmentMetadata.getEndOffset(), "100");
   }
 
   private SegmentZKMetadata getSegmentZKMetadata(String segmentName) {
