@@ -79,6 +79,8 @@ public class QueryGenerator {
 
   private static final List<String> AGGREGATION_FUNCTIONS =
       Arrays.asList("SUM", "MIN", "MAX", "AVG", "COUNT", "DISTINCTCOUNT");
+  private static final List<String> MULTI_STAGE_AGGREGATION_FUNCTIONS =
+      Arrays.asList("SUM", "MIN", "MAX", "AVG", "COUNT");
   private static final Random RANDOM = new Random();
 
   private final Map<String, Set<String>> _columnToValueSet = new HashMap<>();
@@ -93,6 +95,9 @@ public class QueryGenerator {
   private final List<PredicateGenerator> _singleValuePredicateGenerators =
       Arrays.asList(new SingleValueComparisonPredicateGenerator(), new SingleValueInPredicateGenerator(),
           new SingleValueBetweenPredicateGenerator(), new SingleValueRegexPredicateGenerator());
+  private final List<PredicateGenerator> _multistageSingleValuePredicateGenerators =
+      Arrays.asList(new SingleValueComparisonPredicateGenerator(), new SingleValueInPredicateGenerator(),
+          new SingleValueBetweenPredicateGenerator());
   private final List<PredicateGenerator> _multiValuePredicateGenerators =
       Arrays.asList(new MultiValueComparisonPredicateGenerator(), new MultiValueInPredicateGenerator(),
           new MultiValueBetweenPredicateGenerator());
@@ -100,6 +105,7 @@ public class QueryGenerator {
   private final String _pinotTableName;
   private final String _h2TableName;
   private boolean _skipMultiValuePredicates = false;
+  private boolean _useMultistageEngine = false;
 
   /**
    * Constructor for <code>QueryGenerator</code>.
@@ -301,6 +307,10 @@ public class QueryGenerator {
     _skipMultiValuePredicates = skipMultiValuePredicates;
   }
 
+  public void setUseMultistageEngine(boolean useMultistageEngine) {
+    _useMultistageEngine = useMultistageEngine;
+  }
+
   /**
    * Helper method to pick a random value from the values list.
    *
@@ -312,6 +322,10 @@ public class QueryGenerator {
     return list.get(RANDOM.nextInt(list.size()));
   }
 
+  private List<PredicateGenerator> getSingleValuePredicateGenerators() {
+    return _useMultistageEngine ? _multistageSingleValuePredicateGenerators : _singleValuePredicateGenerators;
+  }
+
   /**
    * Generate one selection or aggregation query.
    *
@@ -336,7 +350,7 @@ public class QueryGenerator {
       if (!_columnToValueList.get(columnName).isEmpty()) {
         if (!_multiValueColumnMaxNumElements.containsKey(columnName)) {
           // Single-value column.
-          predicates.add(pickRandom(_singleValuePredicateGenerators).generatePredicate(columnName));
+          predicates.add(pickRandom(getSingleValuePredicateGenerators()).generatePredicate(columnName));
         } else if (!_skipMultiValuePredicates) {
           // Multi-value column.
           predicates.add(pickRandom(_multiValuePredicateGenerators).generatePredicate(columnName));
@@ -493,12 +507,17 @@ public class QueryGenerator {
       List<String> h2AggregateColumnAndFunctions = new ArrayList<>();
       for (String aggregateColumnAndFunction : _aggregateColumnsAndFunctions) {
         String h2AggregateColumnAndFunction;
-        // Make 'AVG' and 'DISTINCTCOUNT' compatible with H2 SQL query.
-        if (aggregateColumnAndFunction.startsWith("AVG(")) {
-          h2AggregateColumnAndFunction =
-              aggregateColumnAndFunction.replace("AVG(", "AVG(CAST(").replace(")", " AS DOUBLE))");
-        } else if (aggregateColumnAndFunction.startsWith("DISTINCTCOUNT(")) {
+        // Make 'AVG' and
+        if (aggregateColumnAndFunction.startsWith("DISTINCTCOUNT(")) {
+          // make 'DISTINCTCOUNT(..)' compatible with H2 SQL query using 'COUNT(DISTINCT(..)'
           h2AggregateColumnAndFunction = aggregateColumnAndFunction.replace("DISTINCTCOUNT(", "COUNT(DISTINCT ");
+        } else if (AGGREGATION_FUNCTIONS.contains(aggregateColumnAndFunction.substring(0, 3))) {
+          // make AGG functions (SUM, MIN, MAX, AVG) compatible with H2 SQL query.
+          // this is because Pinot queries casts all to double before doing aggregation
+          String aggFunctionName = aggregateColumnAndFunction.substring(0, 3);
+          h2AggregateColumnAndFunction = aggregateColumnAndFunction
+              .replace(aggFunctionName + "(", aggFunctionName + "(CAST(")
+              .replace(")", " AS DOUBLE))");
         } else {
           h2AggregateColumnAndFunction = aggregateColumnAndFunction;
         }
@@ -712,9 +731,12 @@ public class QueryGenerator {
     public Query generateQuery() {
       // Select at most MAX_NUM_SELECTION_COLUMNS columns.
       int projectionColumnCount = Math.min(RANDOM.nextInt(MAX_NUM_SELECTION_COLUMNS) + 1, _columnNames.size());
+      if (_useMultistageEngine) {
+        projectionColumnCount = Math.min(projectionColumnCount, _singleValueColumnNames.size());
+      }
       Set<String> projectionColumns = new HashSet<>();
       while (projectionColumns.size() < projectionColumnCount) {
-        projectionColumns.add(pickRandom(_columnNames));
+        projectionColumns.add(pickRandom(_useMultistageEngine ? _singleValueColumnNames : _columnNames));
       }
 
       // Select at most MAX_NUM_ORDER_BY_COLUMNS columns for ORDER BY clause.
@@ -746,29 +768,56 @@ public class QueryGenerator {
 
     @Override
     public Query generateQuery() {
-      // Generate at most MAX_NUM_AGGREGATION_COLUMNS columns on which to aggregate, map 0 to 'COUNT(*)'.
-      int aggregationColumnCount = RANDOM.nextInt(MAX_NUM_AGGREGATION_COLUMNS + 1);
-      Set<String> aggregationColumnsAndFunctions = new HashSet<>();
-      if (aggregationColumnCount == 0) {
-        aggregationColumnsAndFunctions.add("COUNT(*)");
-      } else {
-        while (aggregationColumnsAndFunctions.size() < aggregationColumnCount) {
-          aggregationColumnsAndFunctions.add(createRandomAggregationFunction());
-        }
-      }
-      // Generate a predicate.
-      PredicateQueryFragment predicate = generatePredicate();
       // Generate at most MAX_NUM_GROUP_BY_COLUMNS columns on which to group.
       int groupColumnCount = Math.min(RANDOM.nextInt(MAX_NUM_GROUP_BY_COLUMNS + 1), _singleValueColumnNames.size());
       Set<String> groupColumns = new HashSet<>();
       while (groupColumns.size() < groupColumnCount) {
         groupColumns.add(pickRandom(_singleValueColumnNames));
       }
+
+      // Generate at most MAX_NUM_AGGREGATION_COLUMNS columns on which to aggregate
+      int aggregationColumnCount = RANDOM.nextInt(MAX_NUM_AGGREGATION_COLUMNS + 1);
+      Set<String> aggregationColumnsAndFunctions = new HashSet<>();
+      boolean isDistinctQuery = false;
+      if (aggregationColumnCount == 0) {
+        // if no aggregation function being randomly generated, pick the group by columns into the select list
+        // this should generate a distinct query using query rewriter.
+        // TODO: noted that we don't support distinct/agg rewrite with only part of the group by columns. change this
+        // test once we support such queries.
+        if (groupColumnCount != 0) {
+          aggregationColumnsAndFunctions.addAll(groupColumns);
+          isDistinctQuery = true;
+        } else {
+          aggregationColumnsAndFunctions.add("COUNT(*)");
+        }
+      } else {
+        while (aggregationColumnsAndFunctions.size() < aggregationColumnCount) {
+          aggregationColumnsAndFunctions.add(createRandomAggregationFunction());
+        }
+      }
+
+      // Generate a predicate.
+      PredicateQueryFragment predicate = generatePredicate();
+
       //Generate a HAVING predicate
       ArrayList<String> arrayOfAggregationColumnsAndFunctions = new ArrayList<>(aggregationColumnsAndFunctions);
-      HavingQueryFragment havingPredicate = generateHavingPredicate(arrayOfAggregationColumnsAndFunctions);
+      HavingQueryFragment havingPredicate;
+      if (isDistinctQuery) {
+        havingPredicate = new HavingQueryFragment(Collections.emptyList(), Collections.emptyList(),
+            Collections.emptyList());
+      } else {
+        havingPredicate = generateHavingPredicate(arrayOfAggregationColumnsAndFunctions);
+      }
+
       // Generate a result limit of at most MAX_RESULT_LIMIT.
-      LimitQueryFragment limit = new LimitQueryFragment(RANDOM.nextInt(MAX_RESULT_LIMIT + 1));
+      LimitQueryFragment limit;
+      if (isDistinctQuery) {
+        // Distinct query must have positive LIMIT
+        limit = new LimitQueryFragment(RANDOM.nextInt(MAX_RESULT_LIMIT) + 1);
+      } else {
+        limit = new LimitQueryFragment(RANDOM.nextInt(MAX_RESULT_LIMIT + 1));
+      }
+
       return new AggregationQuery(arrayOfAggregationColumnsAndFunctions, predicate, groupColumns, havingPredicate,
           limit);
     }
@@ -813,7 +862,8 @@ public class QueryGenerator {
     }
 
     private String createRandomAggregationFunction() {
-      String aggregationFunction = pickRandom(AGGREGATION_FUNCTIONS);
+      String aggregationFunction = pickRandom(_useMultistageEngine ? MULTI_STAGE_AGGREGATION_FUNCTIONS
+          : AGGREGATION_FUNCTIONS);
       String aggregationColumn;
       switch (aggregationFunction) {
         // "COUNT" and "DISTINCTCOUNT" support all single-value columns.
