@@ -34,6 +34,7 @@ import org.apache.pinot.common.utils.config.TableConfigUtils;
 import org.apache.pinot.core.common.MinionConstants;
 import org.apache.pinot.core.minion.PinotTaskConfig;
 import org.apache.pinot.minion.MinionContext;
+import org.apache.pinot.minion.event.MinionProgressObserver;
 import org.apache.pinot.plugin.minion.tasks.SegmentConversionResult;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentIndexCreationDriverImpl;
 import org.apache.pinot.segment.local.segment.readers.GenericRowRecordReader;
@@ -220,6 +221,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, "testTable_OFFLINE");
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
@@ -247,6 +249,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
@@ -275,6 +278,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
@@ -304,6 +308,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
@@ -337,6 +342,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME_WITH_PARTITIONING);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600468000000");
@@ -369,6 +375,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME_WITH_SORTED_COL);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
@@ -397,6 +404,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME_EPOCH_HOURS);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
@@ -426,6 +434,7 @@ public class RealtimeToOfflineSegmentsTaskExecutorTest {
 
     RealtimeToOfflineSegmentsTaskExecutor realtimeToOfflineSegmentsTaskExecutor =
         new RealtimeToOfflineSegmentsTaskExecutor(null, null);
+    realtimeToOfflineSegmentsTaskExecutor.setMinionEventObserver(new MinionProgressObserver());
     Map<String, String> configs = new HashMap<>();
     configs.put(MinionConstants.TABLE_NAME_KEY, TABLE_NAME_SDF);
     configs.put(MinionConstants.RealtimeToOfflineSegmentsTask.WINDOW_START_MS_KEY, "1600473600000");
