@@ -281,26 +281,26 @@ public class KafkaPartitionLevelConsumerTest {
           consumer.fetchMessages(new LongMsgOffset(0), new LongMsgOffset(NUM_MSG_PRODUCED_PER_PARTITION), 10000);
       Assert.assertEquals(batch1.getMessageCount(), 500);
       for (int i = 0; i < batch1.getMessageCount(); i++) {
-        final byte[] msg = (byte[]) batch1.getMessageAtIndex(i);
+        final byte[] msg = (byte[]) batch1.getStreamMessage(i).getValue();
         Assert.assertEquals(new String(msg), "sample_msg_" + i);
-        Assert.assertNull(batch1.getMetadataAtIndex(i));
+        Assert.assertNotNull(batch1.getMetadataAtIndex(i));
       }
       // Test second half batch
       final MessageBatch batch2 =
           consumer.fetchMessages(new LongMsgOffset(500), new LongMsgOffset(NUM_MSG_PRODUCED_PER_PARTITION), 10000);
       Assert.assertEquals(batch2.getMessageCount(), 500);
       for (int i = 0; i < batch2.getMessageCount(); i++) {
-        final byte[] msg = (byte[]) batch2.getMessageAtIndex(i);
+        final byte[] msg = (byte[]) batch2.getStreamMessage(i).getValue();
         Assert.assertEquals(new String(msg), "sample_msg_" + (500 + i));
-        Assert.assertNull(batch1.getMetadataAtIndex(i));
+        Assert.assertNotNull(batch1.getMetadataAtIndex(i));
       }
       // Some random range
       final MessageBatch batch3 = consumer.fetchMessages(new LongMsgOffset(10), new LongMsgOffset(35), 10000);
       Assert.assertEquals(batch3.getMessageCount(), 25);
       for (int i = 0; i < batch3.getMessageCount(); i++) {
-        final byte[] msg = (byte[]) batch3.getMessageAtIndex(i);
+        final byte[] msg = (byte[]) batch3.getStreamMessage(i).getValue();
         Assert.assertEquals(new String(msg), "sample_msg_" + (10 + i));
-        Assert.assertNull(batch1.getMetadataAtIndex(i));
+        Assert.assertNotNull(batch1.getMetadataAtIndex(i));
       }
     }
   }
@@ -342,7 +342,7 @@ public class KafkaPartitionLevelConsumerTest {
       for (int i = 0; i < batch1.getMessageCount(); i++) {
         final RowMetadata metadata = batch1.getMetadataAtIndex(i);
         Assert.assertNotNull(metadata);
-        Assert.assertEquals(metadata.getIngestionTimeMs(), TIMESTAMP + i);
+        Assert.assertEquals(metadata.getRecordIngestionTimeMs(), TIMESTAMP + i);
       }
       // Test second half batch
       final MessageBatch batch2 =
@@ -351,7 +351,7 @@ public class KafkaPartitionLevelConsumerTest {
       for (int i = 0; i < batch2.getMessageCount(); i++) {
         final RowMetadata metadata = batch2.getMetadataAtIndex(i);
         Assert.assertNotNull(metadata);
-        Assert.assertEquals(metadata.getIngestionTimeMs(), TIMESTAMP + (500 + i));
+        Assert.assertEquals(metadata.getRecordIngestionTimeMs(), TIMESTAMP + (500 + i));
       }
       // Some random range
       final MessageBatch batch3 = consumer.fetchMessages(new LongMsgOffset(10), new LongMsgOffset(35), 10000);
@@ -359,7 +359,7 @@ public class KafkaPartitionLevelConsumerTest {
       for (int i = 0; i < batch3.getMessageCount(); i++) {
         final RowMetadata metadata = batch3.getMetadataAtIndex(i);
         Assert.assertNotNull(metadata);
-        Assert.assertEquals(metadata.getIngestionTimeMs(), TIMESTAMP + (10 + i));
+        Assert.assertEquals(metadata.getRecordIngestionTimeMs(), TIMESTAMP + (10 + i));
       }
     }
   }
@@ -388,7 +388,7 @@ public class KafkaPartitionLevelConsumerTest {
     MessageBatch batch1 = consumer.fetchMessages(new LongMsgOffset(0), new LongMsgOffset(400), 10000);
     Assert.assertEquals(batch1.getMessageCount(), 200);
     for (int i = 0; i < batch1.getMessageCount(); i++) {
-      byte[] msg = (byte[]) batch1.getMessageAtIndex(i);
+      byte[] msg = (byte[]) batch1.getStreamMessage(i).getValue();
       Assert.assertEquals(new String(msg), "sample_msg_" + (i + 200));
     }
     Assert.assertEquals(batch1.getOffsetOfNextBatch().toString(), "400");
@@ -400,7 +400,7 @@ public class KafkaPartitionLevelConsumerTest {
     MessageBatch batch3 = consumer.fetchMessages(new LongMsgOffset(201), new LongMsgOffset(401), 10000);
     Assert.assertEquals(batch3.getMessageCount(), 200);
     for (int i = 0; i < batch3.getMessageCount(); i++) {
-      byte[] msg = (byte[]) batch3.getMessageAtIndex(i);
+      byte[] msg = (byte[]) batch3.getStreamMessage(i).getValue();
       Assert.assertEquals(new String(msg), "sample_msg_" + (i + 201));
     }
     Assert.assertEquals(batch3.getOffsetOfNextBatch().toString(), "401");
@@ -408,7 +408,7 @@ public class KafkaPartitionLevelConsumerTest {
     MessageBatch batch4 = consumer.fetchMessages(new LongMsgOffset(0), null, 10000);
     Assert.assertEquals(batch4.getMessageCount(), 500);
     for (int i = 0; i < batch4.getMessageCount(); i++) {
-      byte[] msg = (byte[]) batch4.getMessageAtIndex(i);
+      byte[] msg = (byte[]) batch4.getStreamMessage(i).getValue();
       Assert.assertEquals(new String(msg), "sample_msg_" + (i + 200));
     }
     Assert.assertEquals(batch4.getOffsetOfNextBatch().toString(), "700");
