@@ -31,7 +31,7 @@ import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.segment.local.segment.index.column.PhysicalColumnIndexContainer;
 import org.apache.pinot.segment.local.segment.index.loader.columnminmaxvalue.ColumnMinMaxValueGeneratorMode;
 import org.apache.pinot.segment.local.segment.store.TextIndexUtils;
-import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
+import org.apache.pinot.segment.spi.compression.ChunkCompressionType;
 import org.apache.pinot.segment.spi.creator.SegmentVersion;
 import org.apache.pinot.segment.spi.index.creator.H3IndexConfig;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderRegistry;
@@ -40,14 +40,14 @@ import org.apache.pinot.spi.config.table.BloomFilterConfig;
 import org.apache.pinot.spi.config.table.FSTType;
 import org.apache.pinot.spi.config.table.FieldConfig;
 import org.apache.pinot.spi.config.table.IndexingConfig;
+import org.apache.pinot.spi.config.table.JsonIndexConfig;
 import org.apache.pinot.spi.config.table.StarTreeIndexConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
-import org.apache.pinot.spi.config.table.TimestampIndexGranularity;
-import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
-import org.apache.pinot.spi.config.table.ingestion.TransformConfig;
+import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.ReadMode;
+import org.apache.pinot.spi.utils.TimestampIndexUtils;
 
 
 /**
@@ -57,6 +57,7 @@ public class IndexLoadingConfig {
   private static final int DEFAULT_REALTIME_AVG_MULTI_VALUE_COUNT = 2;
   public static final String READ_MODE_KEY = "readMode";
 
+  private InstanceDataManagerConfig _instanceDataManagerConfig = null;
   private ReadMode _readMode = ReadMode.DEFAULT_MODE;
   private List<String> _sortedColumns = Collections.emptyList();
   private Set<String> _invertedIndexColumns = new HashSet<>();
@@ -65,16 +66,18 @@ public class IndexLoadingConfig {
   private Set<String> _textIndexColumns = new HashSet<>();
   private Set<String> _fstIndexColumns = new HashSet<>();
   private FSTType _fstIndexType = FSTType.LUCENE;
-  private Set<String> _jsonIndexColumns = new HashSet<>();
+  private Map<String, JsonIndexConfig> _jsonIndexConfigs = new HashMap<>();
   private Map<String, H3IndexConfig> _h3IndexConfigs = new HashMap<>();
   private Set<String> _noDictionaryColumns = new HashSet<>(); // TODO: replace this by _noDictionaryConfig.
-  private Map<String, String> _noDictionaryConfig = new HashMap<>();
-  private Set<String> _varLengthDictionaryColumns = new HashSet<>();
+  private final Map<String, String> _noDictionaryConfig = new HashMap<>();
+  private final Set<String> _varLengthDictionaryColumns = new HashSet<>();
   private Set<String> _onHeapDictionaryColumns = new HashSet<>();
+  private Set<String> _forwardIndexDisabledColumns = new HashSet<>();
   private Map<String, BloomFilterConfig> _bloomFilterConfigs = new HashMap<>();
   private boolean _enableDynamicStarTreeCreation;
   private List<StarTreeIndexConfig> _starTreeIndexConfigs;
   private boolean _enableDefaultStarTree;
+  private Map<String, ChunkCompressionType> _compressionConfigs = new HashMap<>();
 
   private SegmentVersion _segmentVersion;
   private ColumnMinMaxValueGeneratorMode _columnMinMaxValueGeneratorMode = ColumnMinMaxValueGeneratorMode.DEFAULT_MODE;
@@ -89,17 +92,44 @@ public class IndexLoadingConfig {
   private Map<String, Map<String, String>> _columnProperties = new HashMap<>();
 
   private TableConfig _tableConfig;
+  private Schema _schema;
+
+  private String _tableDataDir;
   private String _segmentDirectoryLoader;
+  private String _segmentTier;
 
   private String _instanceId;
+  private Map<String, Map<String, String>> _instanceTierConfigs;
 
-  public IndexLoadingConfig(InstanceDataManagerConfig instanceDataManagerConfig, TableConfig tableConfig) {
+  /**
+   * NOTE: This step might modify the passed in table config and schema.
+   */
+  public IndexLoadingConfig(InstanceDataManagerConfig instanceDataManagerConfig, TableConfig tableConfig,
+      @Nullable Schema schema) {
     extractFromInstanceConfig(instanceDataManagerConfig);
-    extractFromTableConfig(tableConfig);
-    _tableConfig = tableConfig;
+    extractFromTableConfigAndSchema(tableConfig, schema);
   }
 
-  private void extractFromTableConfig(TableConfig tableConfig) {
+  @VisibleForTesting
+  public IndexLoadingConfig(InstanceDataManagerConfig instanceDataManagerConfig, TableConfig tableConfig) {
+    this(instanceDataManagerConfig, tableConfig, null);
+  }
+
+  @VisibleForTesting
+  public IndexLoadingConfig() {
+  }
+
+  public InstanceDataManagerConfig getInstanceDataManagerConfig() {
+    return _instanceDataManagerConfig;
+  }
+
+  private void extractFromTableConfigAndSchema(TableConfig tableConfig, @Nullable Schema schema) {
+    if (schema != null) {
+      TimestampIndexUtils.applyTimestampIndex(tableConfig, schema);
+    }
+    _tableConfig = tableConfig;
+    _schema = schema;
+
     IndexingConfig indexingConfig = tableConfig.getIndexingConfig();
     String tableReadMode = indexingConfig.getLoadMode();
     if (tableReadMode != null) {
@@ -116,9 +146,18 @@ public class IndexLoadingConfig {
       _invertedIndexColumns.addAll(invertedIndexColumns);
     }
 
-    List<String> jsonIndexColumns = indexingConfig.getJsonIndexColumns();
-    if (jsonIndexColumns != null) {
-      _jsonIndexColumns.addAll(jsonIndexColumns);
+    // Ignore jsonIndexColumns when jsonIndexConfigs is configured
+    Map<String, JsonIndexConfig> jsonIndexConfigs = indexingConfig.getJsonIndexConfigs();
+    if (jsonIndexConfigs != null) {
+      _jsonIndexConfigs = jsonIndexConfigs;
+    } else {
+      List<String> jsonIndexColumns = indexingConfig.getJsonIndexColumns();
+      if (jsonIndexColumns != null) {
+        _jsonIndexConfigs = new HashMap<>();
+        for (String jsonIndexColumn : jsonIndexColumns) {
+          _jsonIndexConfigs.put(jsonIndexColumn, new JsonIndexConfig());
+        }
+      }
     }
 
     List<String> rangeIndexColumns = indexingConfig.getRangeIndexColumns();
@@ -153,36 +192,11 @@ public class IndexLoadingConfig {
       }
     }
 
+    extractCompressionConfigs(tableConfig);
     extractTextIndexColumnsFromTableConfig(tableConfig);
     extractFSTIndexColumnsFromTableConfig(tableConfig);
     extractH3IndexConfigsFromTableConfig(tableConfig);
-
-    Map<String, List<TimestampIndexGranularity>> timestampIndexConfigs =
-        SegmentGeneratorConfig.extractTimestampIndexConfigsFromTableConfig(tableConfig);
-    if (!timestampIndexConfigs.isEmpty()) {
-      // Apply transform function and range index to the timestamp with granularity columns
-      IngestionConfig ingestionConfig = tableConfig.getIngestionConfig();
-      if (ingestionConfig == null) {
-        ingestionConfig = new IngestionConfig();
-        tableConfig.setIngestionConfig(ingestionConfig);
-      }
-      List<TransformConfig> transformConfigs = ingestionConfig.getTransformConfigs();
-      if (transformConfigs == null) {
-        transformConfigs = new ArrayList<>();
-        ingestionConfig.setTransformConfigs(transformConfigs);
-      }
-      for (Map.Entry<String, List<TimestampIndexGranularity>> entry : timestampIndexConfigs.entrySet()) {
-        String column = entry.getKey();
-        for (TimestampIndexGranularity granularity : entry.getValue()) {
-          String columnNameWithGranularity =
-              TimestampIndexGranularity.getColumnNameWithGranularity(column, granularity);
-          TransformConfig transformConfig = new TransformConfig(columnNameWithGranularity,
-              TimestampIndexGranularity.getTransformExpression(column, granularity));
-          transformConfigs.add(transformConfig);
-          _rangeIndexColumns.add(columnNameWithGranularity);
-        }
-      }
-    }
+    extractForwardIndexDisabledColumnsFromTableConfig(tableConfig);
 
     Map<String, String> noDictionaryConfig = indexingConfig.getNoDictionaryConfig();
     if (noDictionaryConfig != null) {
@@ -215,6 +229,28 @@ public class IndexLoadingConfig {
     }
   }
 
+  /**
+   * Extracts compressionType for each column. Populates a map containing column name as key and compression type as
+   * value. This map will only contain the compressionType overrides, and it does not correspond to the default value
+   * of compressionType (derived using SegmentColumnarIndexCreator.getColumnCompressionType())  used for a column.
+   * Note that only RAW forward index columns will be populated in this map.
+   * @param tableConfig table config
+   */
+  private void extractCompressionConfigs(TableConfig tableConfig) {
+    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
+    if (fieldConfigList == null) {
+      return;
+    }
+
+    for (FieldConfig fieldConfig : fieldConfigList) {
+      String column = fieldConfig.getName();
+      if (fieldConfig.getCompressionCodec() != null) {
+        ChunkCompressionType compressionType = ChunkCompressionType.valueOf(fieldConfig.getCompressionCodec().name());
+        _compressionConfigs.put(column, compressionType);
+      }
+    }
+  }
+
   /**
    * Text index creation info for each column is specified
    * using {@link FieldConfig} model of indicating per column
@@ -267,6 +303,8 @@ public class IndexLoadingConfig {
     if (instanceDataManagerConfig == null) {
       return;
     }
+
+    _instanceDataManagerConfig = instanceDataManagerConfig;
     _instanceId = instanceDataManagerConfig.getInstanceId();
 
     ReadMode instanceReadMode = instanceDataManagerConfig.getReadMode();
@@ -295,9 +333,28 @@ public class IndexLoadingConfig {
   }
 
   /**
-   * For tests only.
+   * Forward index disabled info for each column is specified
+   * using {@link FieldConfig} model of indicating per column
+   * encoding and indexing information. Since IndexLoadingConfig
+   * is created from TableConfig, we extract the no forward index info
+   * from fieldConfigList in TableConfig via the properties bag.
+   * @param tableConfig table config
    */
-  public IndexLoadingConfig() {
+  private void extractForwardIndexDisabledColumnsFromTableConfig(TableConfig tableConfig) {
+    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
+    if (fieldConfigList != null) {
+      for (FieldConfig fieldConfig : fieldConfigList) {
+        Map<String, String> fieldConfigProperties = fieldConfig.getProperties();
+        if (fieldConfigProperties != null) {
+          boolean forwardIndexDisabled = Boolean.parseBoolean(
+              fieldConfigProperties.getOrDefault(FieldConfig.FORWARD_INDEX_DISABLED,
+                  FieldConfig.DEFAULT_FORWARD_INDEX_DISABLED));
+          if (forwardIndexDisabled) {
+            _forwardIndexDisabledColumns.add(fieldConfig.getName());
+          }
+        }
+      }
+    }
   }
 
   public ReadMode getReadMode() {
@@ -315,6 +372,19 @@ public class IndexLoadingConfig {
     return _sortedColumns;
   }
 
+  /**
+   * For tests only.
+   */
+  @VisibleForTesting
+  public void setSortedColumn(String sortedColumn) {
+    if (sortedColumn != null) {
+      _sortedColumns = new ArrayList<>();
+      _sortedColumns.add(sortedColumn);
+    } else {
+      _sortedColumns = Collections.emptyList();
+    }
+  }
+
   public Set<String> getInvertedIndexColumns() {
     return _invertedIndexColumns;
   }
@@ -347,8 +417,8 @@ public class IndexLoadingConfig {
     return _fstIndexColumns;
   }
 
-  public Set<String> getJsonIndexColumns() {
-    return _jsonIndexColumns;
+  public Map<String, JsonIndexConfig> getJsonIndexConfigs() {
+    return _jsonIndexConfigs;
   }
 
   public Map<String, H3IndexConfig> getH3IndexConfigs() {
@@ -371,6 +441,24 @@ public class IndexLoadingConfig {
     _invertedIndexColumns = invertedIndexColumns;
   }
 
+  /**
+   * For tests only.
+   * Used by segmentPreProcessorTest to set raw columns.
+   */
+  @VisibleForTesting
+  public void setNoDictionaryColumns(Set<String> noDictionaryColumns) {
+    _noDictionaryColumns = noDictionaryColumns;
+  }
+
+  /**
+   * For tests only.
+   * Used by segmentPreProcessorTest to set compression configs.
+   */
+  @VisibleForTesting
+  public void setCompressionConfigs(Map<String, ChunkCompressionType> compressionConfigs) {
+    _compressionConfigs = compressionConfigs;
+  }
+
   /**
    * For tests only.
    */
@@ -402,7 +490,14 @@ public class IndexLoadingConfig {
 
   @VisibleForTesting
   public void setJsonIndexColumns(Set<String> jsonIndexColumns) {
-    _jsonIndexColumns = jsonIndexColumns;
+    if (jsonIndexColumns != null) {
+      _jsonIndexConfigs = new HashMap<>();
+      for (String jsonIndexColumn : jsonIndexColumns) {
+        _jsonIndexConfigs.put(jsonIndexColumn, new JsonIndexConfig());
+      }
+    } else {
+      _jsonIndexConfigs = null;
+    }
   }
 
   @VisibleForTesting
@@ -420,11 +515,32 @@ public class IndexLoadingConfig {
     _onHeapDictionaryColumns = onHeapDictionaryColumns;
   }
 
+  /**
+   * For tests only.
+   */
+  @VisibleForTesting
+  public void setForwardIndexDisabledColumns(Set<String> forwardIndexDisabledColumns) {
+    _forwardIndexDisabledColumns =
+        forwardIndexDisabledColumns == null ? Collections.emptySet() : forwardIndexDisabledColumns;
+  }
+
   public Set<String> getNoDictionaryColumns() {
     return _noDictionaryColumns;
   }
 
-  public Map<String, String> getnoDictionaryConfig() {
+  /**
+   * Populates a map containing column name as key and compression type as value. This map will only contain the
+   * compressionType overrides, and it does not correspond to the default value of compressionType (derived using
+   * SegmentColumnarIndexCreator.getColumnCompressionType())  used for a column. Note that only RAW forward index
+   * columns will be populated in this map.
+   *
+   * @return a map containing column name as key and compressionType as value.
+   */
+  public Map<String, ChunkCompressionType> getCompressionConfigs() {
+    return _compressionConfigs;
+  }
+
+  public Map<String, String> getNoDictionaryConfig() {
     return _noDictionaryConfig;
   }
 
@@ -436,6 +552,10 @@ public class IndexLoadingConfig {
     return _onHeapDictionaryColumns;
   }
 
+  public Set<String> getForwardIndexDisabledColumns() {
+    return _forwardIndexDisabledColumns;
+  }
+
   public Map<String, BloomFilterConfig> getBloomFilterConfigs() {
     return _bloomFilterConfigs;
   }
@@ -504,6 +624,11 @@ public class IndexLoadingConfig {
     return _tableConfig;
   }
 
+  @Nullable
+  public Schema getSchema() {
+    return _schema;
+  }
+
   @VisibleForTesting
   public void setTableConfig(TableConfig tableConfig) {
     _tableConfig = tableConfig;
@@ -523,4 +648,28 @@ public class IndexLoadingConfig {
   public String getInstanceId() {
     return _instanceId;
   }
+
+  public void setTableDataDir(String tableDataDir) {
+    _tableDataDir = tableDataDir;
+  }
+
+  public String getTableDataDir() {
+    return _tableDataDir;
+  }
+
+  public void setSegmentTier(String segmentTier) {
+    _segmentTier = segmentTier;
+  }
+
+  public String getSegmentTier() {
+    return _segmentTier;
+  }
+
+  public void setInstanceTierConfigs(Map<String, Map<String, String>> tierConfigs) {
+    _instanceTierConfigs = tierConfigs;
+  }
+
+  public Map<String, Map<String, String>> getInstanceTierConfigs() {
+    return _instanceTierConfigs;
+  }
 }
