@@ -19,7 +19,6 @@
 package org.apache.pinot.segment.local.segment.index.loader.columnminmaxvalue;
 
 import com.google.common.base.Preconditions;
-import java.io.FileOutputStream;
 import java.util.HashSet;
 import java.util.Set;
 import org.apache.commons.configuration.PropertiesConfiguration;
@@ -31,28 +30,31 @@ import org.apache.pinot.segment.local.segment.index.readers.IntDictionary;
 import org.apache.pinot.segment.local.segment.index.readers.LongDictionary;
 import org.apache.pinot.segment.local.segment.index.readers.StringDictionary;
 import org.apache.pinot.segment.spi.ColumnMetadata;
-import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
+import org.apache.pinot.segment.spi.SegmentMetadata;
 import org.apache.pinot.segment.spi.memory.PinotDataBuffer;
 import org.apache.pinot.segment.spi.store.ColumnIndexType;
 import org.apache.pinot.segment.spi.store.SegmentDirectory;
+import org.apache.pinot.segment.spi.utils.SegmentMetadataUtils;
 import org.apache.pinot.spi.data.Schema;
 
 import static org.apache.pinot.spi.data.FieldSpec.DataType;
 
 
 public class ColumnMinMaxValueGenerator {
-  private final SegmentMetadataImpl _segmentMetadata;
-  private final PropertiesConfiguration _segmentProperties;
+  private final SegmentMetadata _segmentMetadata;
   private final SegmentDirectory.Writer _segmentWriter;
   private final ColumnMinMaxValueGeneratorMode _columnMinMaxValueGeneratorMode;
 
+  // NOTE: _segmentProperties shouldn't be used when checking whether min/max value need to be generated because at that
+  //       time _segmentMetadata might not be loaded from a local file
+  private PropertiesConfiguration _segmentProperties;
+
   private boolean _minMaxValueAdded;
 
-  public ColumnMinMaxValueGenerator(SegmentMetadataImpl segmentMetadata, SegmentDirectory.Writer segmentWriter,
+  public ColumnMinMaxValueGenerator(SegmentMetadata segmentMetadata, SegmentDirectory.Writer segmentWriter,
       ColumnMinMaxValueGeneratorMode columnMinMaxValueGeneratorMode) {
     _segmentMetadata = segmentMetadata;
     _segmentWriter = segmentWriter;
-    _segmentProperties = segmentMetadata.getPropertiesConfiguration();
     _columnMinMaxValueGeneratorMode = columnMinMaxValueGeneratorMode;
   }
 
@@ -68,10 +70,13 @@ public class ColumnMinMaxValueGenerator {
   public void addColumnMinMaxValue()
       throws Exception {
     Preconditions.checkState(_columnMinMaxValueGeneratorMode != ColumnMinMaxValueGeneratorMode.NONE);
+    _segmentProperties = SegmentMetadataUtils.getPropertiesConfiguration(_segmentMetadata);
     for (String column : getColumnsToAddMinMaxValue()) {
       addColumnMinMaxValueForColumn(column);
     }
-    saveMetadata();
+    if (_minMaxValueAdded) {
+      SegmentMetadataUtils.savePropertiesConfiguration(_segmentProperties);
+    }
   }
 
   private Set<String> getColumnsToAddMinMaxValue() {
@@ -98,7 +103,7 @@ public class ColumnMinMaxValueGenerator {
   private boolean needAddColumnMinMaxValueForColumn(String columnName) {
     ColumnMetadata columnMetadata = _segmentMetadata.getColumnMetadataFor(columnName);
     return columnMetadata.hasDictionary() && columnMetadata.getMinValue() == null
-        && columnMetadata.getMaxValue() == null;
+        && columnMetadata.getMaxValue() == null && !columnMetadata.isMinMaxValueInvalid();
   }
 
   private void addColumnMinMaxValueForColumn(String columnName)
@@ -164,15 +169,4 @@ public class ColumnMinMaxValueGenerator {
 
     _minMaxValueAdded = true;
   }
-
-  private void saveMetadata()
-      throws Exception {
-    if (_minMaxValueAdded) {
-      // Commons Configuration 1.10 does not support file path containing '%'.
-      // Explicitly providing the output stream for the file bypasses the problem.
-      try (FileOutputStream fileOutputStream = new FileOutputStream(_segmentProperties.getFile())) {
-        _segmentProperties.save(fileOutputStream);
-      }
-    }
-  }
 }
