@@ -31,7 +31,7 @@ import org.apache.pinot.plugin.inputformat.avro.AvroRecordReader;
 import org.apache.pinot.segment.local.indexsegment.immutable.ImmutableSegmentLoader;
 import org.apache.pinot.segment.local.segment.creator.SegmentTestUtils;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentIndexCreationDriverImpl;
-import org.apache.pinot.segment.local.segment.readers.IntermediateSegmentRecordReader;
+import org.apache.pinot.segment.local.segment.readers.PinotSegmentRecordReader;
 import org.apache.pinot.segment.spi.ColumnMetadata;
 import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
@@ -43,6 +43,7 @@ import org.apache.pinot.spi.config.table.ColumnPartitionConfig;
 import org.apache.pinot.spi.config.table.SegmentPartitionConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
+import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.TimeGranularitySpec;
@@ -97,7 +98,6 @@ public class IntermediateSegmentTest {
     segmentGeneratorConfig.setTableName("testTable");
     segmentGeneratorConfig.setSegmentName(SEGMENT_NAME);
     segmentGeneratorConfig.setOutDir(INDEX_DIR.getAbsolutePath());
-    segmentGeneratorConfig.setSkipTimeValueCheck(true);
     segmentGeneratorConfig.setInvertedIndexCreationColumns(Arrays.asList("column6", "column7"));
 
     IndexSegment segmentFromIntermediateSegment = buildSegmentFromIntermediateSegment(segmentGeneratorConfig);
@@ -162,15 +162,16 @@ public class IntermediateSegmentTest {
 
     // Ingest data.
     ingestDataToIntermediateSegment(segmentGeneratorConfig, intermediateSegment);
-    IntermediateSegmentRecordReader intermediateSegmentRecordReader =
-        new IntermediateSegmentRecordReader(intermediateSegment);
+    PinotSegmentRecordReader recordReader = new PinotSegmentRecordReader();
+    recordReader.init(intermediateSegment);
 
     // Build the segment from intermediate segment.
     SegmentIndexCreationDriverImpl driver = new SegmentIndexCreationDriverImpl();
-    driver.init(segmentGeneratorConfig, intermediateSegmentRecordReader);
+    driver.init(segmentGeneratorConfig, recordReader);
     driver.build();
 
     // Destroy intermediate segment after the segment creation.
+    recordReader.close();
     intermediateSegment.destroy();
 
     return ImmutableSegmentLoader.load(new File(INDEX_DIR, segmentName), ReadMode.heap);
@@ -229,10 +230,19 @@ public class IntermediateSegmentTest {
   private static TableConfig createTableConfig(String inputFile) {
     TableConfig tableConfig;
     if (AVRO_DATA_SV.equals(inputFile)) {
+      // The segment generation code in SegmentColumnarIndexCreator will throw
+      // exception if start and end time in time column are not in acceptable
+      // range. For this test, we first need to fix the input avro data
+      // to have the time column values in allowed range. Until then, the check
+      // is explicitly disabled
+      IngestionConfig ingestionConfig = new IngestionConfig();
+      ingestionConfig.setSegmentTimeValueCheck(false);
+      ingestionConfig.setRowTimeValueCheck(false);
       tableConfig =
           new TableConfigBuilder(TableType.OFFLINE).setTableName("testTable").setTimeColumnName("daysSinceEpoch")
               .setInvertedIndexColumns(Arrays.asList("column6", "column7", "column11", "column17", "column18"))
-              .setSegmentPartitionConfig(getSegmentPartitionConfig()).build();
+              .setSegmentPartitionConfig(getSegmentPartitionConfig())
+              .setIngestionConfig(ingestionConfig).build();
     } else {
       tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName("testTable").build();
     }
