@@ -23,17 +23,21 @@ import it.unimi.dsi.fastutil.floats.FloatOpenHashSet;
 import it.unimi.dsi.fastutil.ints.IntOpenHashSet;
 import it.unimi.dsi.fastutil.longs.LongOpenHashSet;
 import java.io.File;
+import java.math.BigDecimal;
 import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Random;
 import java.util.Set;
+import java.util.TreeSet;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang.RandomStringUtils;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentDictionaryCreator;
 import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.memory.PinotDataBuffer;
 import org.apache.pinot.spi.data.DimensionFieldSpec;
-import org.apache.pinot.spi.data.FieldSpec;
+import org.apache.pinot.spi.data.FieldSpec.DataType;
+import org.apache.pinot.spi.data.MetricFieldSpec;
+import org.apache.pinot.spi.utils.BigDecimalUtils;
 import org.apache.pinot.spi.utils.ByteArray;
 import org.apache.pinot.spi.utils.BytesUtils;
 import org.testng.Assert;
@@ -51,6 +55,7 @@ public class ImmutableDictionaryTest {
   private static final String LONG_COLUMN_NAME = "longColumn";
   private static final String FLOAT_COLUMN_NAME = "floatColumn";
   private static final String DOUBLE_COLUMN_NAME = "doubleColumn";
+  private static final String BIG_DECIMAL_COLUMN_NAME = "bigDecimalColumn";
   private static final String STRING_COLUMN_NAME = "stringColumn";
   private static final String BYTES_COLUMN_NAME = "bytesColumn";
   private static final int NUM_VALUES = 1000;
@@ -61,6 +66,8 @@ public class ImmutableDictionaryTest {
   private long[] _longValues;
   private float[] _floatValues;
   private double[] _doubleValues;
+  private BigDecimal[] _bigDecimalValues;
+  private int _bigDecimalByteLength;
   private String[] _stringValues;
   private ByteArray[] _bytesValues;
 
@@ -99,6 +106,15 @@ public class ImmutableDictionaryTest {
     _doubleValues = doubleSet.toDoubleArray();
     Arrays.sort(_doubleValues);
 
+    TreeSet<BigDecimal> bigDecimalSet = new TreeSet<>();
+    while (bigDecimalSet.size() < NUM_VALUES) {
+      BigDecimal bigDecimal = BigDecimal.valueOf(RANDOM.nextDouble());
+      _bigDecimalByteLength = Math.max(_bigDecimalByteLength, BigDecimalUtils.byteSize(bigDecimal));
+      bigDecimalSet.add(bigDecimal);
+    }
+    _bigDecimalValues = bigDecimalSet.toArray(new BigDecimal[0]);
+    Arrays.sort(_bigDecimalValues);
+
     Set<String> stringSet = new HashSet<>();
     while (stringSet.size() < NUM_VALUES) {
       stringSet.add(RandomStringUtils.random(RANDOM.nextInt(MAX_STRING_LENGTH)).replace('\0', ' '));
@@ -115,35 +131,44 @@ public class ImmutableDictionaryTest {
     _bytesValues = bytesSet.toArray(new ByteArray[NUM_VALUES]);
     Arrays.sort(_bytesValues);
 
-    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(_intValues,
-        new DimensionFieldSpec(INT_COLUMN_NAME, FieldSpec.DataType.INT, true), TEMP_DIR)) {
-      dictionaryCreator.build();
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(
+        new DimensionFieldSpec(INT_COLUMN_NAME, DataType.INT, true), TEMP_DIR)) {
+      dictionaryCreator.build(_intValues);
     }
 
-    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(_longValues,
-        new DimensionFieldSpec(LONG_COLUMN_NAME, FieldSpec.DataType.LONG, true), TEMP_DIR)) {
-      dictionaryCreator.build();
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(
+        new DimensionFieldSpec(LONG_COLUMN_NAME, DataType.LONG, true), TEMP_DIR)) {
+      dictionaryCreator.build(_longValues);
     }
 
-    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(_floatValues,
-        new DimensionFieldSpec(FLOAT_COLUMN_NAME, FieldSpec.DataType.FLOAT, true), TEMP_DIR)) {
-      dictionaryCreator.build();
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(
+        new DimensionFieldSpec(FLOAT_COLUMN_NAME, DataType.FLOAT, true), TEMP_DIR)) {
+      dictionaryCreator.build(_floatValues);
     }
 
-    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(_doubleValues,
-        new DimensionFieldSpec(DOUBLE_COLUMN_NAME, FieldSpec.DataType.DOUBLE, true), TEMP_DIR)) {
-      dictionaryCreator.build();
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(
+        new DimensionFieldSpec(DOUBLE_COLUMN_NAME, DataType.DOUBLE, true), TEMP_DIR)) {
+      dictionaryCreator.build(_doubleValues);
     }
 
-    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(_stringValues,
-        new DimensionFieldSpec(STRING_COLUMN_NAME, FieldSpec.DataType.STRING, true), TEMP_DIR)) {
-      dictionaryCreator.build();
+    // Note: BigDecimalDictionary requires setting useVarLengthDictionary to true.
+    boolean useVarLengthDictionary = true;
+    MetricFieldSpec bigDecimalMetricField = new MetricFieldSpec(BIG_DECIMAL_COLUMN_NAME, DataType.BIG_DECIMAL);
+    bigDecimalMetricField.setSingleValueField(true);
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(bigDecimalMetricField, TEMP_DIR,
+        useVarLengthDictionary)) {
+      dictionaryCreator.build(_bigDecimalValues);
+    }
+
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(
+        new DimensionFieldSpec(STRING_COLUMN_NAME, DataType.STRING, true), TEMP_DIR)) {
+      dictionaryCreator.build(_stringValues);
       _numBytesPerStringValue = dictionaryCreator.getNumBytesPerEntry();
     }
 
-    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(_bytesValues,
-        new DimensionFieldSpec(BYTES_COLUMN_NAME, FieldSpec.DataType.BYTES, true), TEMP_DIR)) {
-      dictionaryCreator.build();
+    try (SegmentDictionaryCreator dictionaryCreator = new SegmentDictionaryCreator(
+        new DimensionFieldSpec(BYTES_COLUMN_NAME, DataType.BYTES, true), TEMP_DIR)) {
+      dictionaryCreator.build(_bytesValues);
       assertEquals(dictionaryCreator.getNumBytesPerEntry(), BYTES_LENGTH);
     }
   }
@@ -172,7 +197,7 @@ public class ImmutableDictionaryTest {
     for (int i = 0; i < NUM_VALUES; i++) {
       assertEquals(intDictionary.get(i), _intValues[i]);
       assertEquals(intDictionary.getIntValue(i), _intValues[i]);
-      assertEquals(intDictionary.getLongValue(i), (long) _intValues[i]);
+      assertEquals(intDictionary.getLongValue(i), _intValues[i]);
       assertEquals(intDictionary.getFloatValue(i), (float) _intValues[i]);
       assertEquals(intDictionary.getDoubleValue(i), (double) _intValues[i]);
       Assert.assertEquals(Integer.parseInt(intDictionary.getStringValue(i)), _intValues[i]);
@@ -188,9 +213,8 @@ public class ImmutableDictionaryTest {
   @Test
   public void testLongDictionary()
       throws Exception {
-    try (LongDictionary longDictionary = new LongDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, LONG_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)),
-        NUM_VALUES)) {
+    try (LongDictionary longDictionary = new LongDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, LONG_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES)) {
       testLongDictionary(longDictionary);
     }
   }
@@ -198,9 +222,8 @@ public class ImmutableDictionaryTest {
   @Test
   public void testOnHeapLongDictionary()
       throws Exception {
-    try (OnHeapLongDictionary onHeapLongDictionary = new OnHeapLongDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, LONG_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)),
-        NUM_VALUES)) {
+    try (OnHeapLongDictionary onHeapLongDictionary = new OnHeapLongDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, LONG_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES)) {
       testLongDictionary(onHeapLongDictionary);
     }
   }
@@ -225,9 +248,8 @@ public class ImmutableDictionaryTest {
   @Test
   public void testFloatDictionary()
       throws Exception {
-    try (FloatDictionary floatDictionary = new FloatDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, FLOAT_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)),
-        NUM_VALUES)) {
+    try (FloatDictionary floatDictionary = new FloatDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, FLOAT_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES)) {
       testFloatDictionary(floatDictionary);
     }
   }
@@ -235,9 +257,9 @@ public class ImmutableDictionaryTest {
   @Test
   public void testOnHeapFloatDictionary()
       throws Exception {
-    try (OnHeapFloatDictionary onHeapFloatDictionary = new OnHeapFloatDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, FLOAT_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)),
-        NUM_VALUES)) {
+    try (OnHeapFloatDictionary onHeapFloatDictionary = new OnHeapFloatDictionary(
+        PinotDataBuffer.mapReadOnlyBigEndianFile(
+            new File(TEMP_DIR, FLOAT_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES)) {
       testFloatDictionary(onHeapFloatDictionary);
     }
   }
@@ -262,9 +284,8 @@ public class ImmutableDictionaryTest {
   @Test
   public void testDoubleDictionary()
       throws Exception {
-    try (DoubleDictionary doubleDictionary = new DoubleDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, DOUBLE_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)),
-        NUM_VALUES)) {
+    try (DoubleDictionary doubleDictionary = new DoubleDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, DOUBLE_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES)) {
       testDoubleDictionary(doubleDictionary);
     }
   }
@@ -272,9 +293,9 @@ public class ImmutableDictionaryTest {
   @Test
   public void testOnHeapDoubleDictionary()
       throws Exception {
-    try (OnHeapDoubleDictionary onHeapDoubleDictionary = new OnHeapDoubleDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, DOUBLE_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)),
-        NUM_VALUES)) {
+    try (OnHeapDoubleDictionary onHeapDoubleDictionary = new OnHeapDoubleDictionary(
+        PinotDataBuffer.mapReadOnlyBigEndianFile(
+            new File(TEMP_DIR, DOUBLE_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES)) {
       testDoubleDictionary(onHeapDoubleDictionary);
     }
   }
@@ -296,12 +317,51 @@ public class ImmutableDictionaryTest {
     }
   }
 
+  @Test
+  public void testBigDecimalDictionary()
+      throws Exception {
+    try (BigDecimalDictionary bigDecimalDictionary = new BigDecimalDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, BIG_DECIMAL_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES,
+        _bigDecimalByteLength)) {
+      testBigDecimalDictionary(bigDecimalDictionary);
+    }
+  }
+
+  @Test
+  public void testOnHeapBigDecimalDictionary()
+      throws Exception {
+    try (OnHeapBigDecimalDictionary onHeapBigDecimalDictionary = new OnHeapBigDecimalDictionary(
+        PinotDataBuffer.mapReadOnlyBigEndianFile(
+            new File(TEMP_DIR, BIG_DECIMAL_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES,
+        _bigDecimalByteLength)) {
+      testBigDecimalDictionary(onHeapBigDecimalDictionary);
+    }
+  }
+
+  private void testBigDecimalDictionary(BaseImmutableDictionary bigDecimalDictionary) {
+    for (int i = 0; i < NUM_VALUES; i++) {
+      assertEquals(bigDecimalDictionary.get(i), _bigDecimalValues[i]);
+      assertEquals(bigDecimalDictionary.getIntValue(i), _bigDecimalValues[i].intValue());
+      assertEquals(bigDecimalDictionary.getLongValue(i), _bigDecimalValues[i].longValue());
+      assertEquals(bigDecimalDictionary.getFloatValue(i), _bigDecimalValues[i].floatValue());
+      assertEquals(bigDecimalDictionary.getDoubleValue(i), _bigDecimalValues[i].doubleValue());
+      assertEquals(bigDecimalDictionary.getBigDecimalValue(i), _bigDecimalValues[i]);
+      Assert.assertEquals(new BigDecimal(bigDecimalDictionary.getStringValue(i)), _bigDecimalValues[i]);
+
+      assertEquals(bigDecimalDictionary.indexOf(String.valueOf(_bigDecimalValues[i])), i);
+
+      BigDecimal randomBigDecimal = BigDecimal.valueOf(RANDOM.nextDouble());
+      assertEquals(bigDecimalDictionary.insertionIndexOf(String.valueOf(randomBigDecimal)),
+          Arrays.binarySearch(_bigDecimalValues, randomBigDecimal));
+    }
+  }
+
   @Test
   public void testStringDictionary()
       throws Exception {
-    try (StringDictionary stringDictionary = new StringDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, STRING_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES,
-        _numBytesPerStringValue, (byte) 0)) {
+    try (StringDictionary stringDictionary = new StringDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, STRING_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES, _numBytesPerStringValue,
+        (byte) 0)) {
       testStringDictionary(stringDictionary);
     }
   }
@@ -309,8 +369,9 @@ public class ImmutableDictionaryTest {
   @Test
   public void testOnHeapStringDictionary()
       throws Exception {
-    try (OnHeapStringDictionary onHeapStringDictionary = new OnHeapStringDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, STRING_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES,
+    try (OnHeapStringDictionary onHeapStringDictionary = new OnHeapStringDictionary(
+        PinotDataBuffer.mapReadOnlyBigEndianFile(
+            new File(TEMP_DIR, STRING_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES,
         _numBytesPerStringValue, (byte) 0)) {
       testStringDictionary(onHeapStringDictionary);
     }
@@ -332,13 +393,22 @@ public class ImmutableDictionaryTest {
   @Test
   public void testBytesDictionary()
       throws Exception {
-    try (BytesDictionary bytesDictionary = new BytesDictionary(PinotDataBuffer
-        .mapReadOnlyBigEndianFile(new File(TEMP_DIR, BYTES_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES,
-        BYTES_LENGTH)) {
+    try (BytesDictionary bytesDictionary = new BytesDictionary(PinotDataBuffer.mapReadOnlyBigEndianFile(
+        new File(TEMP_DIR, BYTES_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES, BYTES_LENGTH)) {
       testBytesDictionary(bytesDictionary);
     }
   }
 
+  @Test
+  public void testOnHeapBytesDictionary()
+      throws Exception {
+    try (OnHeapBytesDictionary onHeapBytesDictionary = new OnHeapBytesDictionary(
+        PinotDataBuffer.mapReadOnlyBigEndianFile(
+            new File(TEMP_DIR, BYTES_COLUMN_NAME + V1Constants.Dict.FILE_EXTENSION)), NUM_VALUES, BYTES_LENGTH)) {
+      testBytesDictionary(onHeapBytesDictionary);
+    }
+  }
+
   private void testBytesDictionary(BaseImmutableDictionary bytesDictionary) {
     for (int i = 0; i < NUM_VALUES; i++) {
       assertEquals(bytesDictionary.get(i), _bytesValues[i].getBytes());
