@@ -21,6 +21,7 @@ package org.apache.pinot.segment.spi.index.reader;
 import java.io.Closeable;
 import java.math.BigDecimal;
 import javax.annotation.Nullable;
+import org.apache.pinot.segment.spi.compression.ChunkCompressionType;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.utils.BigDecimalUtils;
 
@@ -49,6 +50,24 @@ public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends
    */
   DataType getStoredType();
 
+  /**
+   * Returns the compression type (if valid). Only valid for RAW forward index columns implemented in
+   * BaseChunkForwardIndexReader.
+   * @return
+   */
+  default ChunkCompressionType getCompressionType() {
+    return null;
+  }
+
+  /**
+   * Returns the length of the longest entry. Only valid for RAW forward index columns implemented in
+   * BaseChunkForwardIndexReader. Returns -1 otherwise.
+   * @return
+   */
+  default int getLengthOfLongestEntry() {
+    return -1;
+  }
+
   /**
    * Creates a new {@link ForwardIndexReaderContext} of the reader which can be used to accelerate the reads.
    * NOTE: Caller is responsible for closing the returned reader context.
@@ -322,16 +341,16 @@ public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends
           values[i] = BigDecimal.valueOf(getDouble(docIds[i], context));
         }
         break;
-      case STRING:
-        for (int i = 0; i < length; i++) {
-          values[i] = new BigDecimal(getString(docIds[i], context));
-        }
-        break;
       case BIG_DECIMAL:
         for (int i = 0; i < length; i++) {
           values[i] = getBigDecimal(docIds[i], context);
         }
         break;
+      case STRING:
+        for (int i = 0; i < length; i++) {
+          values[i] = new BigDecimal(getString(docIds[i], context));
+        }
+        break;
       case BYTES:
         for (int i = 0; i < length; i++) {
           values[i] = BigDecimalUtils.deserialize(getBytes(docIds[i], context));
@@ -723,6 +742,20 @@ public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends
     }
   }
 
+  /**
+   * Fills the values
+   * @param docIds Array containing the document ids to read
+   * @param length Number of values to read
+   * @param maxNumValuesPerMVEntry Maximum number of values per MV entry
+   * @param values Values to fill
+   * @param context Reader context
+   */
+  default void readValuesMV(int[] docIds, int length, int maxNumValuesPerMVEntry, byte[][][] values, T context) {
+    for (int i = 0; i < length; i++) {
+      values[i] = getBytesMV(docIds[i], context);
+    }
+  }
+
   /**
    * Reads the INT type multi-value at the given document id into the passed in value buffer (the buffer size must be
    * enough to hold all the values for the multi-value entry) and returns the number of values within the multi-value
