@@ -121,8 +121,14 @@ public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig
   // Size of cache that holds errors.
   private static final String ERROR_CACHE_SIZE = "error.cache.size";
 
+  private static final String DELETED_SEGMENTS_CACHE_SIZE = "table.deleted.segments.cache.size";
+  private static final String DELETED_SEGMENTS_CACHE_TTL_MINUTES = "table.deleted.segments.cache.ttl.minutes";
+  private static final String PEER_DOWNLOAD_SCHEME = "peer.download.scheme";
+
   private final static String[] REQUIRED_KEYS = {INSTANCE_ID, INSTANCE_DATA_DIR, READ_MODE};
   private static final long DEFAULT_ERROR_CACHE_SIZE = 100L;
+  private static final int DEFAULT_DELETED_SEGMENTS_CACHE_SIZE = 10_000;
+  private static final int DEFAULT_DELETED_SEGMENTS_CACHE_TTL_MINUTES = 2;
   private PinotConfiguration _instanceDataManagerConfiguration = null;
 
   public HelixInstanceDataManagerConfig(PinotConfiguration serverConfig)
@@ -257,6 +263,23 @@ public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig
         DEFAULT_STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT);
   }
 
+  @Override
+  public int getDeletedSegmentsCacheSize() {
+    return _instanceDataManagerConfiguration.getProperty(DELETED_SEGMENTS_CACHE_SIZE,
+        DEFAULT_DELETED_SEGMENTS_CACHE_SIZE);
+  }
+
+  @Override
+  public int getDeletedSegmentsCacheTtlMinutes() {
+    return _instanceDataManagerConfiguration.getProperty(DELETED_SEGMENTS_CACHE_TTL_MINUTES,
+        DEFAULT_DELETED_SEGMENTS_CACHE_TTL_MINUTES);
+  }
+
+  @Override
+  public String getSegmentPeerDownloadScheme() {
+    return _instanceDataManagerConfiguration.getProperty(PEER_DOWNLOAD_SCHEME);
+  }
+
   @Override
   public String toString() {
     String configString = "";
