@@ -19,9 +19,6 @@
 package org.apache.pinot.server.starter.helix;
 
 import com.google.common.base.Preconditions;
-import java.io.File;
-import java.util.concurrent.locks.Lock;
-import org.apache.commons.io.FileUtils;
 import org.apache.helix.NotificationContext;
 import org.apache.helix.model.Message;
 import org.apache.helix.participant.statemachine.StateModel;
@@ -38,7 +35,6 @@ import org.apache.pinot.core.data.manager.InstanceDataManager;
 import org.apache.pinot.core.data.manager.realtime.LLRealtimeSegmentDataManager;
 import org.apache.pinot.segment.local.data.manager.SegmentDataManager;
 import org.apache.pinot.segment.local.data.manager.TableDataManager;
-import org.apache.pinot.segment.local.utils.SegmentLocks;
 import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.slf4j.Logger;
@@ -131,7 +127,7 @@ public class SegmentOnlineOfflineStateModelFactory extends StateModelFactory<Sta
       String realtimeTableName = message.getResourceName();
       String segmentName = message.getPartitionName();
       try {
-        _instanceDataManager.removeSegment(realtimeTableName, segmentName);
+        _instanceDataManager.offloadSegment(realtimeTableName, segmentName);
       } catch (Exception e) {
         _logger.error("Caught exception in state transition from CONSUMING -> OFFLINE for resource: {}, partition: {}",
             realtimeTableName, segmentName, e);
@@ -186,7 +182,7 @@ public class SegmentOnlineOfflineStateModelFactory extends StateModelFactory<Sta
       String tableNameWithType = message.getResourceName();
       String segmentName = message.getPartitionName();
       try {
-        _instanceDataManager.removeSegment(tableNameWithType, segmentName);
+        _instanceDataManager.offloadSegment(tableNameWithType, segmentName);
       } catch (Exception e) {
         _logger.error("Caught exception in state transition from ONLINE -> OFFLINE for resource: {}, partition: {}",
             tableNameWithType, segmentName, e);
@@ -200,22 +196,11 @@ public class SegmentOnlineOfflineStateModelFactory extends StateModelFactory<Sta
       _logger.info("SegmentOnlineOfflineStateModel.onBecomeDroppedFromOffline() : " + message);
       String tableNameWithType = message.getResourceName();
       String segmentName = message.getPartitionName();
-
-      // This method might modify the file on disk. Use segment lock to prevent race condition
-      Lock segmentLock = SegmentLocks.getSegmentLock(tableNameWithType, segmentName);
       try {
-        segmentLock.lock();
-
-        File segmentDir = _instanceDataManager.getSegmentDataDirectory(tableNameWithType, segmentName);
-        if (segmentDir.exists()) {
-          FileUtils.deleteQuietly(segmentDir);
-          _logger.info("Deleted segment directory {}", segmentDir);
-        }
+        _instanceDataManager.deleteSegment(tableNameWithType, segmentName);
       } catch (final Exception e) {
-        _logger.error("Cannot delete the segment : " + segmentName + " from local directory!\n" + e.getMessage(), e);
+        _logger.error("Cannot drop the segment : " + segmentName + " from server!\n" + e.getMessage(), e);
         Utils.rethrowException(e);
-      } finally {
-        segmentLock.unlock();
       }
     }
 
