@@ -72,7 +72,8 @@ public abstract class QuickStartBase {
       "examples/batch/dimBaseballTeams",
       "examples/batch/starbucksStores",
       "examples/batch/githubEvents",
-      "examples/batch/githubComplexTypeEvents"
+      "examples/batch/githubComplexTypeEvents",
+      "examples/batch/billing",
   };
 
   protected static final Map<String, String> DEFAULT_STREAM_TABLE_DIRECTORIES = ImmutableMap.<String, String>builder()
@@ -103,7 +104,17 @@ public abstract class QuickStartBase {
     return this;
   }
 
-  /** @return Table name if specified by command line argument -bootstrapTableDir; otherwise, default. */
+  /* @return bootstrap path if specified by command line argument -bootstrapTableDir; otherwise, null. */
+  public String getBootstrapDataDir() {
+    return _bootstrapDataDirs != null && _bootstrapDataDirs.length == 1 ? _bootstrapDataDirs[0] : null;
+  }
+
+  /** @return Table name specified by command line argument -bootstrapTableDir */
+  public String getTableName() {
+    return Paths.get(getBootstrapDataDir()).getFileName().toString();
+  }
+
+  /** @return Table name if specified by input bootstrap directory. */
   public String getTableName(String bootstrapDataDir) {
     return Paths.get(bootstrapDataDir).getFileName().toString();
   }
@@ -146,16 +157,19 @@ public abstract class QuickStartBase {
   protected List<QuickstartTableRequest> bootstrapOfflineTableDirectories(File quickstartTmpDir)
       throws IOException {
     List<QuickstartTableRequest> quickstartTableRequests = new ArrayList<>();
-    for (String directory : getDefaultBatchTableDirectories()) {
-      String tableName = getTableName(directory);
-      File baseDir = new File(quickstartTmpDir, tableName);
-      File dataDir = new File(baseDir, "rawdata");
-      Preconditions.checkState(dataDir.mkdirs());
-      if (useDefaultBootstrapTableDir()) {
+    if (useDefaultBootstrapTableDir()) {
+      for (String directory : getDefaultBatchTableDirectories()) {
+        String tableName = getTableName(directory);
+        File baseDir = new File(quickstartTmpDir, tableName);
+        File dataDir = new File(baseDir, "rawdata");
+        Preconditions.checkState(dataDir.mkdirs());
         copyResourceTableToTmpDirectory(directory, tableName, baseDir, dataDir, false);
-      } else {
-        copyFilesystemTableToTmpDirectory(directory, tableName, baseDir);
+        quickstartTableRequests.add(new QuickstartTableRequest(baseDir.getAbsolutePath()));
       }
+    } else {
+      String tableName = getTableName();
+      File baseDir = new File(quickstartTmpDir, tableName);
+      copyFilesystemTableToTmpDirectory(getBootstrapDataDir(), tableName, baseDir);
       quickstartTableRequests.add(new QuickstartTableRequest(baseDir.getAbsolutePath()));
     }
     return quickstartTableRequests;
@@ -385,7 +399,7 @@ public abstract class QuickStartBase {
         case "meetupRsvp":
           kafkaStarter.createTopic("meetupRSVPEvents", KafkaStarterUtils.getTopicCreationProps(10));
           printStatus(Quickstart.Color.CYAN, "***** Starting meetup data stream and publishing to Kafka *****");
-          MeetupRsvpStream meetupRSVPProvider = new MeetupRsvpStream();
+          MeetupRsvpStream meetupRSVPProvider = new MeetupRsvpStream(true);
           meetupRSVPProvider.run();
           Runtime.getRuntime().addShutdownHook(new Thread(() -> {
             try {
