@@ -201,11 +201,13 @@ public class StartControllerCommand extends AbstractBaseAdminCommand implements
     Map<String, Object> properties = new HashMap<>();
     if (_configFileName != null) {
       properties.putAll(PinotConfigUtils.generateControllerConf(_configFileName));
+      ControllerConf conf = new ControllerConf(properties);
+
       // Override the zkAddress and clusterName to ensure ServiceManager is connecting to the right Zookeeper and
       // Cluster.
       // Configs existence is already verified.
-      _zkAddress = properties.get(ControllerConf.ZK_STR).toString();
-      _clusterName = properties.get(ControllerConf.HELIX_CLUSTER_NAME).toString();
+      _zkAddress = conf.getZkStr();
+      _clusterName = conf.getHelixClusterName();
     } else {
       if (_controllerHost == null) {
         _controllerHost = NetUtils.getHostAddress();
