@@ -80,7 +80,7 @@ public class PerfBenchmarkRunner extends AbstractBaseCommand implements Command
       description = "Comma separated bloom filter columns to be created (non-batch load).")
   private String _bloomFilterColumns;
 
-  @CommandLine.Option(names = {"-help", "-h", "--h", "--help"}, required = false, help = true,
+  @CommandLine.Option(names = {"-help", "-h", "--h", "--help"}, required = false, usageHelp = true,
       description = "Print this message.")
   private boolean _help = false;
 
@@ -168,7 +168,8 @@ public class PerfBenchmarkRunner extends AbstractBaseCommand implements Command
       List<String> invertedIndexColumns, List<String> bloomFilterColumns)
       throws Exception {
     boolean tableConfigured = false;
-    File[] segments = new File(dataDir, tableName).listFiles();
+    // Skip BaseTableDataManager._resourceTmpDir
+    File[] segments = new File(dataDir, tableName).listFiles((dir, name) -> !name.equals("tmp"));
     Preconditions.checkNotNull(segments);
     for (File segment : segments) {
       if (segment.isDirectory()) {
@@ -192,12 +193,11 @@ public class PerfBenchmarkRunner extends AbstractBaseCommand implements Command
       throws Exception {
     PerfBenchmarkRunner perfBenchmarkRunner = new PerfBenchmarkRunner();
     CommandLine commandLine = new CommandLine(perfBenchmarkRunner);
-    commandLine.parseArgs(args);
-
-    if (perfBenchmarkRunner._help) {
-      perfBenchmarkRunner.printUsage();
-    } else {
-      perfBenchmarkRunner.execute();
+    CommandLine.ParseResult result = commandLine.parseArgs(args);
+    if (result.isUsageHelpRequested() || result.matchedArgs().size() == 0) {
+      commandLine.usage(System.out);
+      return;
     }
+    commandLine.execute();
   }
 }
